#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_mysqlflexibleserver
version_added: "2.7.0"
short_description: Manage MySQL Flexible Server instance
description:
    - Create, update and delete instance of MySQL Flexible Server.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    name:
        description:
            - The name of the server.
        required: True
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    sku:
        description:
            - The SKU (pricing tier) of the server.
        type: dict
        suboptions:
            name:
                description:
                    - The name of the sku, e.g. Standard_D32s_v3.
                type: str
                required: true
            tier:
                description:
                    - The tier of the particular SKU,
                type: str
                choices:
                    - Burstable
                    - GeneralPurpose
                    - MemoryOptimized
                required: true
    storage:
        description:
            - Storage Profile properties of a server.
        type: dict
        suboptions:
            storage_size_gb:
                description:
                    - Max storage size allowed for a server.
                type: int
            iops:
                description:
                    - Storage IOPS for a server.
                type: int
            auto_grow:
                description:
                    - Enable Storage Auto Grow or not.
                type: str
                choices:
                    - Disabled
                    - Enabled
    version:
        description:
            - Server version.
        type: str
        choices:
            - '5.7'
            - '8.0.21'
    administrator_login:
        description:
            - The administrator's login name of a server.
            - Can only be specified when the server is being created (and is required for creation).
        type: str
    administrator_login_password:
        description:
            - The password of the administrator login.
        type: str
    availability_zone:
        description:
            - Availability Zone information of the server.
        type: str
    restore_point_in_time:
        description:
            - Restore point creation time (ISO8601 format), specifying the time to restore from.
            - Required when C(source_server_resource_id) is specified.
        type: str
    source_server_resource_id:
        description:
            - The source MySQL server id.
            - Required when C(restore_point_in_time) is specified.
        type: str
    backup:
        description:
            - Backup related properties of a server.
        type: dict
        suboptions:
            geo_redundant_backup:
                description:
                    - Whether or not geo redundant backup is enabled.
                type: str
                choices:
                    - Enabled
                    - Disabled
            backup_retention_days:
                description:
                    - Backup retention days for the server.
                type: int
    network:
        description:
            - Network related properties of a server.
        type: dict
        suboptions:
            delegated_subnet_resource_id:
                description:
                    - Delegated subnet resource id used to setup vnet for a server.
                type: str
            private_dns_zone_resource_id:
                description:
                    - Private DNS zone resource id.
                type: str
    high_availability:
        description:
            - High availability related properties of a server.
        type: dict
        suboptions:
            mode:
                description:
                    - High availability mode for a server.
                type: str
                choices:
                    - Disabled
                    - ZoneRedundant
                    - SameZone
            standby_availability_zone:
                description:
                    - Vailability zone of the standby server.
                type: str
    status:
        description:
            Set the server state.
        type: str
        choices:
            - restart
            - start
            - stop
            - failover
    state:
        description:
            - Assert the state of the MySQL Flexible Server. Use C(present) to create or update a server and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create mysql flexible server
  azure_rm_mysqlflexibleserver:
    resource_group: "{{ resource_group }}"
    name: postflexible{{ rpfx }}
    sku:
      name: Standard_D2ds_v4
      tier: GeneralPurpose
    administrator_login: azureuser
    administrator_login_password: Fred@0329
    location: northeurope
    version: 5.7
    storage:
      storage_size_gb: 128
      iops: 500
      auto_grow: Enabled
    high_availability:
      mode: ZoneRedundant
      standby_availability_zone: 3
    backup:
      backup_retention_days: 7
      geo_redundant_backup: Disabled
    availability_zone: 1

- name: Restore MySQL flexible server from a point-in-time backup
  azure_rm_mysqlflexibleserver:
    resource_group: "{{ resource_group }}"
    name: postflexible-restore{{ rpfx }}
    source_server_resource_id: "{{ mysql_flexible_server_id }}"
    restore_point_in_time: "2026-02-26T02:27:14Z"
'''

RETURN = '''
servers:
    description:
        - The facts of the flexible servers.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxxx-xxxx/resourceGroups/testRG/providers/Microsoft.DBforMySQL/flexibleServers/server01
        resource_group:
            description:
                - Resource group name.
            returned: always
            type: str
            sample: testRG
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: server01
        location:
            description:
                - The location the resource resides in.
            returned: always
            type: str
            sample: eastus
        sku:
            description:
                - The SKU of the server.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - The name of the SKU.
                    returned: always
                    type: str
                    sample: Standard_D32s_v3
                tier:
                    description:
                        - The tier of the particular SKU.
                    returned: always
                    type: str
                    sample: GeneralPurpose
        storage:
            description:
                - Storage related properties of a server.
            type: complex
            returned: always
            contains:
                storage_size_gb:
                    description:
                        - Max storage size allowed for a server.
                    returned: always
                    type: int
                    sample: 128000
                iops:
                    description:
                        - Storage IOPS for a server.
                    returned: always
                    type: int
                    sample: 684
                auto_grow:
                    description:
                        - Enable Storage Auto Grow or not.
                    returned: always
                    type: str
                    sample: Disabled
        availability_zone:
            description:
                - Availability Zone information of the server.
            returned: always
            type: str
            sample: 1
        administrator_login_password:
            description:
                - The administrator's login name of a server.
            returned: always
            type: str
            sample: serveradmin
        backup:
            description:
                - Backup related properties of a server.
            type: complex
            contains:
                backup_retention_days:
                    description:
                        - Backup retention days for the server.
                    type: int
                    returned: always
                    sample: 7
                geo_redundant_backup:
                    description:
                        - Whether or not geo redundant backup is enabled.
                    type: str
                    returned: always
                    sample: Disabled
        version:
            description:
                - Server version.
            returned: always
            type: str
            sample: "5.7"
        status:
            description:
                - Set server state.
            type: str
            choices:
                - restart
                - start
                - stop
                - failover
        state:
            description:
                - A state of a server that is visible to user.
            returned: always
            type: str
            sample: Ready
        fully_qualified_domain_name:
            description:
                - The fully qualified domain name of a server.
            returned: always
            type: str
            sample: myabdud1223.mys.database.azure.com
        high_availability:
            description:
                - High availability related properties of a server.
            type: complex
            returned: always
            contains:
                mode:
                    description:
                        - High availability mode for a server.
                    type: str
                    sample: Disabled
                    returned: always
                standby_availability_zone:
                    description:
                        - Availability zone of the standby server.
                    type: str
                    sample: Availability zone of the standby server.
                    returned: always
        network:
            description:
                - Network related properties of a server.
            type: complex
            returned: always
            contains:
                delegated_subnet_resource_id:
                    description:
                        - Delegated subnet resource id used to setup vnet for a server.
                    type: str
                    sample: null
                    returned: always
                private_dns_zone_resource_id:
                    description:
                        - Private DNS zone resource id.
                    type: str
                    sample: null
                    returned: always
        restore_point_in_time:
            description:
                - Restore point creation time (ISO8601 format), specifying the time to restore from.
            type: str
            returned: always
            sample: null
        source_server_resource_id:
            description:
                - The source MySQL server id.
            type: str
            returned: always
            sample: null
        tags:
            description:
                - Tags assigned to the resource. Dictionary of string:string pairs.
            type: dict
            returned: always
            sample: { tag1: abc }
        type:
            description:
                - The type of the resource.
            type: str
            returned: always
            sample: Microsoft.DBforMySQL/flexibleServers
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass

storage_spec = dict(
    storage_size_gb=dict(
        type='int'
    ),
    iops=dict(
        type='int'
    ),
    auto_grow=dict(
        type='str',
        choices=['Disabled', 'Enabled']
    )
)


high_availability_spec = dict(
    mode=dict(type='str', choices=["Disabled", "ZoneRedundant", "SameZone"]),
    standby_availability_zone=dict(type='str')
)


sku_spec = dict(
    name=dict(type='str', required=True),
    tier=dict(type='str', required=True, choices=["Burstable", "GeneralPurpose", "MemoryOptimized"])
)


backup_spec = dict(
    backup_retention_days=dict(type='int'),
    geo_redundant_backup=dict(type='str', choices=["Enabled", "Disabled"])
)


network_spec = dict(
    delegated_subnet_resource_id=dict(type='str'),
    private_dns_zone_resource_id=dict(type='str'),
)


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMMySqlFlexibleServers(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM MySQL Flexible Server resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            sku=dict(
                type='dict',
                options=sku_spec
            ),
            location=dict(
                type='str'
            ),
            version=dict(
                type='str',
                choices=['5.7', '8.0.21']
            ),
            administrator_login=dict(
                type='str'
            ),
            administrator_login_password=dict(
                type='str',
                no_log=True
            ),
            availability_zone=dict(
                type='str'
            ),
            restore_point_in_time=dict(
                type='str'
            ),
            source_server_resource_id=dict(
                type='str'
            ),
            backup=dict(
                type='dict',
                options=backup_spec
            ),
            network=dict(
                type='dict',
                options=network_spec
            ),
            storage=dict(
                type='dict',
                options=storage_spec
            ),
            high_availability=dict(
                type='dict',
                options=high_availability_spec
            ),
            status=dict(
                type='str',
                choices=['restart', 'start', 'stop', 'failover']
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.parameters = dict()
        self.update_parameters = dict()
        self.status = None
        self.tags = None

        self.results = dict(changed=False)
        self.state = None
        self.to_do = Actions.NoAction

        required_together = [['restore_point_in_time', 'source_server_resource_id']]

        super(AzureRMMySqlFlexibleServers, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          required_together=required_together,
                                                          supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == "sku":
                    self.parameters["sku"] = kwargs[key]
                    self.update_parameters["sku"] = kwargs[key]
                elif key == "location":
                    self.parameters["location"] = kwargs[key]
                elif key == "storage":
                    self.parameters["storage"] = kwargs[key]
                    self.update_parameters["storage"] = kwargs[key]
                elif key == "version":
                    self.parameters["version"] = kwargs[key]
                elif key == "administrator_login":
                    self.parameters["administrator_login"] = kwargs[key]
                elif key == "administrator_login_password":
                    self.parameters["administrator_login_password"] = kwargs[key]
                elif key == 'availability_zone':
                    self.parameters['availability_zone'] = kwargs[key]
                elif key == 'source_server_resource_id':
                    self.parameters['source_server_resource_id'] = kwargs[key]
                elif key == 'restore_point_in_time':
                    self.parameters['restore_point_in_time'] = kwargs[key]
                elif key == 'backup':
                    self.parameters['backup'] = kwargs[key]
                    self.update_parameters['backup'] = kwargs[key]
                elif key == 'high_availability':
                    self.parameters['high_availability'] = kwargs[key]
                    self.update_parameters['high_availability'] = kwargs[key]
                elif key == 'network':
                    self.parameters['network'] = kwargs[key]

        self.parameters['tags'] = self.tags

        # If this is a PointInTimeRestore, set create_mode to PointInTimeRestore
        if self.parameters.get('source_server_resource_id') and self.parameters.get('restore_point_in_time'):
            self.parameters['create_mode'] = 'PointInTimeRestore'

        old_response = None
        response = None
        changed = False

        resource_group = self.get_resource_group(self.resource_group)
        if "location" not in self.parameters:
            self.parameters["location"] = resource_group.location

        old_response = self.get_mysqlserver()

        if not old_response:
            if self.state == 'absent':
                self.log("The mysql flexible server didn't exist")
            else:
                changed = True
                self.to_do = Actions.Create
        else:
            self.log("MySQL Flexible Server instance already exists")
            if self.state == 'absent':
                changed = True
                self.to_do = Actions.Delete
            else:
                self.log("Whether the MySQL Flexible Server instance need update")
                update_tags, self.update_parameters['tags'] = self.update_tags(old_response.get('tags'))
                if update_tags:
                    changed = True
                    self.to_do = Actions.Update

                for item in ['sku', 'network', 'storage', 'backup', 'high_availability']:
                    if not self.default_compare({}, self.update_parameters.get(item), old_response[item], '', dict(compare=[])):
                        changed = True
                        self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the MySQL Flexible Server instance")

            if not self.check_mode:
                response = self.create_update_mysqlserver()
                self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("MySQL Flexible Server instance deleted")
            if not self.check_mode:
                self.delete_mysqlserver()
        if self.status is not None:
            if self.status == 'start':
                self.start_mysqlserver()
                changed = True
            elif self.status == 'restart':
                self.restart_mysqlserver()
                changed = True
            elif self.status == 'stop':
                self.stop_mysqlserver()
                changed = True
            elif self.status == 'failover':
                self.failover_mysqlserver()
                changed = True
            else:
                pass
        self.results['changed'] = changed
        self.results['state'] = self.get_mysqlserver()
        return self.results

    def failover_mysqlserver(self):
        '''
        Manual failover MySQL Flexible Server.
        '''
        self.log("Failover MySQL Flexible Server instance {0}".format(self.name))

        try:
            response = self.mysql_flexible_client.servers.begin_failover(resource_group_name=self.resource_group,
                                                                         server_name=self.name)
        except Exception as exc:
            self.fail("Error failover mysql flexible server {0} - {1}".format(self.name, str(exc)))
        return True

    def stop_mysqlserver(self):
        '''
        Stop MySQL Flexible Server.
        '''
        self.log("Stop MySQL Flexible Server instance {0}".format(self.name))

        try:
            response = self.mysql_flexible_client.servers.begin_stop(resource_group_name=self.resource_group,
                                                                     server_name=self.name)
        except Exception as exc:
            self.fail("Error stop mysql flexible server {0} - {1}".format(self.name, str(exc)))
        return True

    def start_mysqlserver(self):
        '''
        Start MySQL Flexible Server.
        '''
        self.log("Start MySQL Flexible Server instance {0}".format(self.name))

        try:
            response = self.mysql_flexible_client.servers.begin_start(resource_group_name=self.resource_group,
                                                                      server_name=self.name)
        except Exception as exc:
            self.fail("Error starting mysql flexible server {0} - {1}".format(self.name, str(exc)))
        return True

    def restart_mysqlserver(self):
        '''
        Restart MySQL Flexible Server.
        '''
        self.log("Restart MySQL Flexible Server instance {0}".format(self.name))

        try:
            response = self.mysql_flexible_client.servers.begin_restart(resource_group_name=self.resource_group,
                                                                        server_name=self.name,
                                                                        parameters=dict(restart_with_failover='Enabled',
                                                                                        max_failover_seconds=20))
        except Exception as exc:
            self.fail("Error restarting mysql flexible server {0} - {1}".format(self.name, str(exc)))
        return True

    def create_update_mysqlserver(self):
        '''
        Creates or updates MySQL Flexible Server with the specified configuration.

        :return: deserialized MySQL Flexible Server instance state dictionary
        '''
        self.log("Creating / Updating the MySQL Flexible Server instance {0}".format(self.name))

        try:
            self.parameters['tags'] = self.tags
            if self.to_do == Actions.Create:
                response = self.mysql_flexible_client.servers.begin_create(resource_group_name=self.resource_group,
                                                                           server_name=self.name,
                                                                           parameters=self.parameters)
            else:
                # structure of parameters for update must be changed
                response = self.mysql_flexible_client.servers.begin_update(resource_group_name=self.resource_group,
                                                                           server_name=self.name,
                                                                           parameters=self.update_parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the MySQL Flexible Server instance.')
            self.fail("Error creating the MySQL Flexible Server instance: {0}".format(str(exc)))
        return self.format_item(response)

    def delete_mysqlserver(self):
        '''
        Deletes specified MySQL Flexible Server instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the MySQL Flexible Server instance {0}".format(self.name))
        try:
            response = self.mysql_flexible_client.servers.begin_delete(resource_group_name=self.resource_group,
                                                                       server_name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the MySQL Flexible Server instance.')
            self.fail("Error deleting the MySQL Flexible Server instance: {0}".format(str(e)))

        return True

    def get_mysqlserver(self):
        '''
        Gets the properties of the specified MySQL Flexible Server.

        :return: deserialized MySQL Flexible Server instance state dictionary
        '''
        self.log("Checking if the MySQL Flexible Server instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mysql_flexible_client.servers.get(resource_group_name=self.resource_group,
                                                              server_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("MySQL Flexible Server instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the MySQL Flexible Server instance.')
        if found is True:
            return self.format_item(response)

        return False

    def format_item(self, item):
        results = dict(
            resource_group=self.parse_resource_to_dict(item.id).get('resource_group'),
            id=item.id,
            name=item.name,
            type=item.type,
            tags=item.tags,
            location=item.location,
            sku=dict(),
            administrator_login=item.administrator_login,
            version=item.version,
            availability_zone=item.availability_zone,
            source_server_resource_id=item.source_server_resource_id,
            restore_point_in_time=item.restore_point_in_time,
            state=item.state,
            fully_qualified_domain_name=item.fully_qualified_domain_name,
            storage=dict(),
            backup=dict(),
            high_availability=dict(),
            network=dict(),
        )
        if item.sku not in [None, 'None']:
            results['sku']['name'] = item.sku.name
            results['sku']['tier'] = item.sku.tier
        else:
            results['sku'] = None
        if item.storage not in [None, 'None']:
            results['storage']['storage_size_gb'] = item.storage.storage_size_gb
            results['storage']['iops'] = item.storage.iops
            results['storage']['auto_grow'] = item.storage.auto_grow
        else:
            results['storage'] = None
        if item.high_availability not in [None, 'None']:
            results['high_availability']['standby_availability_zone'] = item.high_availability.standby_availability_zone
            results['high_availability']['mode'] = item.high_availability.mode
        else:
            results['high_availability'] = None
        if item.backup not in [None, 'None']:
            results['backup']['backup_retention_days'] = item.backup.backup_retention_days
            results['backup']['geo_redundant_backup'] = item.backup.geo_redundant_backup
        else:
            results['backup'] = None
        if item.network not in [None, 'None']:
            results['network']['delegated_subnet_resource_id'] = item.network.delegated_subnet_resource_id
            results['network']['private_dns_zone_resource_id'] = item.network.private_dns_zone_resource_id
        else:
            results['network'] = None
        return results


def main():
    """Main execution"""
    AzureRMMySqlFlexibleServers()


if __name__ == '__main__':
    main()



# Copyright (c) 2024, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: run0
short_description: Systemd's run0
description:
  - This become plugins allows your remote/login user to execute commands as another user using the C(run0) utility.
author:
  - Thomas Sjögren (@konstruktoid)
version_added: '9.0.0'
options:
  become_user:
    description: User you 'become' to execute the task.
    default: root
    ini:
      - section: privilege_escalation
        key: become_user
      - section: run0_become_plugin
        key: user
    vars:
      - name: ansible_become_user
      - name: ansible_run0_user
    env:
      - name: ANSIBLE_BECOME_USER
      - name: ANSIBLE_RUN0_USER
    type: string
  become_exe:
    description: C(run0) executable.
    default: run0
    ini:
      - section: privilege_escalation
        key: become_exe
      - section: run0_become_plugin
        key: executable
    vars:
      - name: ansible_become_exe
      - name: ansible_run0_exe
    env:
      - name: ANSIBLE_BECOME_EXE
      - name: ANSIBLE_RUN0_EXE
    type: string
  become_flags:
    description: Options to pass to C(run0).
    default: ''
    ini:
      - section: privilege_escalation
        key: become_flags
      - section: run0_become_plugin
        key: flags
    vars:
      - name: ansible_become_flags
      - name: ansible_run0_flags
    env:
      - name: ANSIBLE_BECOME_FLAGS
      - name: ANSIBLE_RUN0_FLAGS
    type: string
notes:
  - This plugin only works when a C(polkit) rule is in place.
  - This become plugin does not work when connection pipelining is enabled. With ansible-core 2.19+, using it automatically
    disables pipelining. On ansible-core 2.18 and before, pipelining must explicitly be disabled by the user.
"""

EXAMPLES = r"""
# An example polkit rule that allows the user 'ansible' in the 'wheel' group
# to execute commands using run0 without authentication.
/etc/polkit-1/rules.d/60-run0-fast-user-auth.rules: |-
  polkit.addRule(function(action, subject) {
    if(action.id == "org.freedesktop.systemd1.manage-units" &&
      subject.isInGroup("wheel") &&
      subject.user == "ansible") {
        return polkit.Result.YES;
    }
  });
"""

from re import compile as re_compile

from ansible.module_utils.common.text.converters import to_bytes
from ansible.plugins.become import BecomeBase

ansi_color_codes = re_compile(to_bytes(r"\x1B\[[0-9;]+m"))


class BecomeModule(BecomeBase):
    name = "community.general.run0"

    prompt = "Password: "
    fail = ("==== AUTHENTICATION FAILED ====",)
    success = ("==== AUTHENTICATION COMPLETE ====",)
    require_tty = True  # see https://github.com/ansible-collections/community.general/issues/6932

    # See https://github.com/ansible/ansible/issues/81254,
    # https://github.com/ansible/ansible/pull/78111
    pipelining = False

    @staticmethod
    def remove_ansi_codes(line):
        return ansi_color_codes.sub(b"", line)

    def build_become_command(self, cmd, shell):
        super().build_become_command(cmd, shell)

        if not cmd:
            return cmd

        become = self.get_option("become_exe")
        flags = self.get_option("become_flags")
        user = self.get_option("become_user")

        # SYSTEMD_COLORS=0 stops run0 from emitting terminal control
        # sequences (window title OSC, ANSI reset) around the child
        # command, which would otherwise corrupt the module JSON and
        # break result parsing.
        return f"SYSTEMD_COLORS=0 {become} --user={user} {flags} {self._build_success_command(cmd, shell)}"

    def check_success(self, b_output):
        b_output = self.remove_ansi_codes(b_output)
        return super().check_success(b_output)

    def check_incorrect_password(self, b_output):
        b_output = self.remove_ansi_codes(b_output)
        return super().check_incorrect_password(b_output)

    def check_missing_password(self, b_output):
        b_output = self.remove_ansi_codes(b_output)
        return super().check_missing_password(b_output)



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_user
extends_documentation_fragment:
- cisco.nxos.nxos
author: Peter Sprygada (@privateip)
notes:
- Limited Support for Cisco MDS
short_description: Manage the collection of local users on Nexus devices
description:
- This module provides declarative management of the local usernames configured on
  Cisco Nexus devices.  It allows playbooks to manage either individual usernames
  or the collection of usernames in the current running config.  It also supports
  purging usernames from the configuration that are not explicitly defined.
version_added: 1.0.0
options:
  aggregate:
    description:
    - The set of username objects to be configured on the remote Cisco Nexus device.  The
      list entries can either be the username or a hash of username and properties.  This
      argument is mutually exclusive with the C(name) argument.
    aliases:
    - users
    - collection
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The username to be configured on the remote Cisco Nexus device.  This argument
          accepts a string value and is mutually exclusive with the C(aggregate) argument.
        type: str
      configured_password:
        description:
        - The password to be configured on the network device. The password needs to be
          provided in cleartext and it will be encrypted on the device.
        type: str
      hashed_password:
        description:
        - The hashed password to be configured on the network device. The password needs to
          already be encrypted.
        type: str
      update_password:
        description:
        - Since passwords are encrypted in the device running config, this argument will
          instruct the module when to change the password.  When set to C(always), the
          password will always be updated in the device and when set to C(on_create) the
          password will be updated only if the username is created.
        choices:
        - on_create
        - always
        type: str
      roles:
        description:
        - The C(role) argument configures the role for the username in the device running
          configuration.  The argument accepts a string value defining the role name.  This
          argument does not check if the role has been configured on the device.
        aliases:
        - role
        type: list
        elements: str
      sshkey:
        description:
        - The C(sshkey) argument defines the SSH public key to configure for the username.  This
          argument accepts a valid SSH key value.
        type: str
      state:
        description:
        - The C(state) argument configures the state of the username definition as it
         relates to the device operational configuration.  When set to I(present), the
         username(s) should be configured in the device active configuration and when
         set to I(absent) the username(s) should not be in the device active configuration
        choices:
        - present
        - absent
        type: str
  name:
    description:
    - The username to be configured on the remote Cisco Nexus device.  This argument
      accepts a string value and is mutually exclusive with the C(aggregate) argument.
    type: str
  configured_password:
    description:
    - The password to be configured on the network device. The password needs to be
      provided in cleartext and it will be encrypted on the device.
    type: str
  hashed_password:
    description:
    - The hashed password to be configured on the network device. The password needs to
      already be encrypted.
    type: str
  update_password:
    description:
    - Since passwords are encrypted in the device running config, this argument will
      instruct the module when to change the password.  When set to C(always), the
      password will always be updated in the device and when set to C(on_create) the
      password will be updated only if the username is created.
    default: always
    choices:
    - on_create
    - always
    type: str
  roles:
    description:
    - The C(role) argument configures the role for the username in the device running
      configuration.  The argument accepts a string value defining the role name.  This
      argument does not check if the role has been configured on the device.
    aliases:
    - role
    type: list
    elements: str
  sshkey:
    description:
    - The C(sshkey) argument defines the SSH public key to configure for the username.  This
      argument accepts a valid SSH key value.
    type: str
  purge:
    description:
    - The C(purge) argument instructs the module to consider the resource definition
      absolute.  It will remove any previously configured usernames on the device
      with the exception of the `admin` user which cannot be deleted per nxos constraints.
    type: bool
    default: false
  state:
    description:
    - The C(state) argument configures the state of the username definition as it
      relates to the device operational configuration.  When set to I(present), the
      username(s) should be configured in the device active configuration and when
      set to I(absent) the username(s) should not be in the device active configuration
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
- name: create a new user
  cisco.nxos.nxos_user:
    name: ansible
    sshkey: "{{ lookup('file', '~/.ssh/id_rsa.pub') }}"
    state: present

- name: remove all users except admin
  cisco.nxos.nxos_user:
    purge: true

- name: set multiple users role
  cisco.nxos.nxos_user:
    aggregate:
      - name: netop
      - name: netend
    role: network-operator
  state: present
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - name ansible
    - name ansible password password
"""
import re

from copy import deepcopy
from functools import partial

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
    to_list,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_config,
    load_config,
    run_commands,
)


BUILTIN_ROLES = [
    "network-admin",
    "network-operator",
    "vdc-admin",
    "vdc-operator",
    "priv-15",
    "priv-14",
    "priv-13",
    "priv-12",
    "priv-11",
    "priv-10",
    "priv-9",
    "priv-8",
    "priv-7",
    "priv-6",
    "priv-5",
    "priv-4",
    "priv-3",
    "priv-2",
    "priv-1",
    "priv-0",
    "dev-ops",
]


def get_custom_roles(module):
    return re.findall(
        r"^role name (\S+)",
        get_config(module, flags=["| include '^role name'"]),
        re.M,
    )


def validate_roles(value, module):
    valid_roles = BUILTIN_ROLES + get_custom_roles(module)
    for item in value:
        if item not in valid_roles:
            module.fail_json(msg="invalid role specified")


def map_obj_to_commands(updates, module):
    commands = list()
    update_password = module.params["update_password"]

    for update in updates:
        want, have = update

        def needs_update(x):
            return want.get(x) and (want.get(x) != have.get(x))

        def add(x):
            return commands.append("username %s %s" % (want["name"], x))

        def remove(x):
            return commands.append("no username %s %s" % (want["name"], x))

        def configure_roles():
            if want["roles"]:
                if have:
                    for item in set(have["roles"]).difference(want["roles"]):
                        remove("role %s" % item)

                    for item in set(want["roles"]).difference(have["roles"]):
                        add("role %s" % item)
                else:
                    for item in want["roles"]:
                        add("role %s" % item)

                return True
            return False

        if want["state"] == "absent":
            commands.append("no username %s" % want["name"])
            continue

        roles_configured = False
        if want["state"] == "present" and not have:
            roles_configured = configure_roles()
            if not roles_configured:
                commands.append("username %s" % want["name"])

        if needs_update("configured_password"):
            if update_password == "always" or not have:
                add("password %s" % want["configured_password"])

        if needs_update("hashed_password"):
            if update_password == "always" or not have:
                add("password 5 %s" % want["hashed_password"])

        if needs_update("sshkey"):
            add("sshkey %s" % want["sshkey"])

        if not roles_configured:
            configure_roles()

    return commands


def parse_password(data):
    if not data.get("remote_login"):
        return "<PASSWORD>"


def parse_roles(data):
    configured_roles = None
    if "TABLE_role" in data:
        configured_roles = data.get("TABLE_role")["ROW_role"]

    roles = list()
    if configured_roles:
        for item in to_list(configured_roles):
            roles.append(item["role"])
    return roles


def map_config_to_obj(module):
    out = run_commands(module, [{"command": "show user-account", "output": "json"}])
    data = out[0]

    objects = list()

    for item in to_list(data["TABLE_template"]["ROW_template"]):
        objects.append(
            {
                "name": item["usr_name"],
                "configured_password": parse_password(item),
                "hashed_password": parse_password(item),
                "sshkey": item.get("sshkey_info"),
                "roles": parse_roles(item),
                "state": "present",
            },
        )
    return objects


def get_param_value(key, item, module):
    # if key doesn't exist in the item, get it from module.params
    if not item.get(key):
        value = module.params[key]

    # if key does exist, do a type check on it to validate it
    else:
        value_type = module.argument_spec[key].get("type", "str")
        type_checker = module._CHECK_ARGUMENT_TYPES_DISPATCHER[value_type]
        type_checker(item[key])
        value = item[key]

    return value


def map_params_to_obj(module):
    aggregate = module.params["aggregate"]
    if not aggregate:
        if not module.params["name"] and module.params["purge"]:
            return list()
        elif not module.params["name"]:
            module.fail_json(msg="username is required")
        else:
            collection = [{"name": module.params["name"]}]
    else:
        collection = list()
        for item in aggregate:
            if not isinstance(item, dict):
                collection.append({"name": item})
            elif "name" not in item:
                module.fail_json(msg="name is required")
            else:
                collection.append(item)

    objects = list()

    for item in collection:
        get_value = partial(get_param_value, item=item, module=module)
        item.update(
            {
                "configured_password": get_value("configured_password"),
                "hashed_password": get_value("hashed_password"),
                "sshkey": get_value("sshkey"),
                "roles": get_value("roles"),
                "state": get_value("state"),
            },
        )

        for key, value in item.items():
            if value:
                # validate the param value (if validator func exists)
                validator = globals().get("validate_%s" % key)
                if all((value, validator)):
                    validator(value, module)

        objects.append(item)

    return objects


def update_objects(want, have):
    updates = list()
    for entry in want:
        item = next((i for i in have if i["name"] == entry["name"]), None)
        if all((item is None, entry["state"] == "present")):
            updates.append((entry, {}))
        elif item:
            for key, value in entry.items():
                if value and value != item[key]:
                    updates.append((entry, item))
    return updates


def get_configured_usernames(module):
    config_output = run_commands(
        module,
        [{"command": "show running-config | section ^username", "output": "text"}],
    )
    usernames = set()
    for line in config_output[0].splitlines():
        if line.startswith("username "):
            username = line.split()[1]
            usernames.add(username)
    return usernames


def main():
    """main entry point for module execution"""
    element_spec = dict(
        name=dict(),
        configured_password=dict(no_log=True),
        hashed_password=dict(no_log=True),
        update_password=dict(default="always", choices=["on_create", "always"]),
        roles=dict(type="list", aliases=["role"], elements="str"),
        sshkey=dict(no_log=False),
        state=dict(default="present", choices=["present", "absent"]),
    )

    aggregate_spec = deepcopy(element_spec)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    argument_spec = dict(
        aggregate=dict(
            type="list",
            elements="dict",
            options=aggregate_spec,
            aliases=["collection", "users"],
        ),
        purge=dict(type="bool", default=False),
    )

    argument_spec.update(element_spec)

    mutually_exclusive = [("name", "aggregate"), ("configured_password", "hashed_password")]

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    result = {"changed": False, "warnings": []}

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands(update_objects(want, have), module)

    if module.params["purge"]:
        want_users = set([x["name"] for x in want])
        have_users = get_configured_usernames(module)

        for item in set(have_users).difference(want_users):
            if item != "admin":
                item = item.replace("\\", "\\\\")
                commands.append("no username %s" % item)

    result["commands"] = commands

    # the nxos cli prevents this by rule so capture it and display
    # a nice failure message
    if "no username admin" in commands:
        module.fail_json(msg="cannot delete the `admin` account")

    # check if provided hashed password is infact a hash
    if module.params["hashed_password"] is not None:
        if not re.match(r"^\$5\$......\$.*$", module.params["hashed_password"]):
            module.fail_json(msg="Provided hash is not valid")

    if commands:
        if not module.check_mode:
            responses = load_config(module, commands)
            for resp in responses:
                if resp.lower().startswith("wrong password"):
                    module.fail_json(msg=resp)
                else:
                    result["warnings"].extend(
                        [x[9:] for x in resp.splitlines() if x.startswith("WARNING: ")],
                    )

        result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# Copyright (c) 2019, Sandeep Kasargod <sandeep@vexata.com>
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

HAS_VEXATAPI = True
try:
    from vexatapi.vexata_api_proxy import VexataAPIProxy
except ImportError:
    HAS_VEXATAPI = False

from ansible.module_utils.basic import env_fallback

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


VXOS_VERSION = None


def get_version(iocs_json) -> tuple[int, ...]:
    if not iocs_json:
        raise Exception("Invalid IOC json")
    active = next((x for x in iocs_json if x["mgmtRole"]), None)
    if active is None:
        raise Exception("Unable to detect active IOC")
    ver = active["swVersion"]
    if ver[0] != "v":
        raise Exception("Illegal version string")
    ver = ver[1 : ver.find("-")]
    ver = map(int, ver.split("."))
    return tuple(ver)


def get_array(module: AnsibleModule):
    """Return storage array object or fail"""
    global VXOS_VERSION
    array = module.params["array"]
    user = module.params.get("user", None)
    password = module.params.get("password", None)
    validate = module.params.get("validate_certs")

    if not HAS_VEXATAPI:
        module.fail_json(msg="vexatapi library is required for this module. To install, use `pip install vexatapi`")

    if user and password:
        system = VexataAPIProxy(array, user, password, verify_cert=validate)
    else:
        module.fail_json(
            msg="The user/password are required to be passed in to "
            "the module as arguments or by setting the "
            "VEXATA_USER and VEXATA_PASSWORD environment variables."
        )
    try:
        if system.test_connection():
            VXOS_VERSION = get_version(system.iocs())
            return system
        else:
            module.fail_json(msg="Test connection to array failed.")
    except Exception as e:
        module.fail_json(msg=f"Vexata API access failed: {e}")


def argument_spec() -> dict[str, t.Any]:
    """Return standard base dictionary used for the argument_spec argument in AnsibleModule"""
    return dict(
        array=dict(type="str", required=True),
        user=dict(type="str", fallback=(env_fallback, ["VEXATA_USER"])),
        password=dict(type="str", no_log=True, fallback=(env_fallback, ["VEXATA_PASSWORD"])),
        validate_certs=dict(type="bool", required=False, default=False),
    )


def required_together() -> list[list[str]]:
    """Return the default list used for the required_together argument to AnsibleModule"""
    return [["user", "password"]]


def size_to_MiB(size: str) -> int:
    """Convert a '<integer>[MGT]' string to MiB, return -1 on error."""
    quant = size[:-1]
    exponent = size[-1]
    if not quant.isdigit() or exponent not in "MGT":
        return -1
    quant_int = int(quant)
    if exponent == "G":
        quant_int <<= 10
    elif exponent == "T":
        quant_int <<= 20
    return quant_int



# (c) 2017, Tennis Smith, https://github.com/gamename
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
    name: profile_roles
    type: aggregate
    short_description: adds timing information to roles
    description:
        - This callback module provides profiling for ansible roles.
    requirements:
      - whitelisting in configuration
    options:
      summary_only:
        description:
          - Only show summary, not individual task profiles.
            Especially usefull in combination with C(DISPLAY_SKIPPED_HOSTS=false) and/or C(ANSIBLE_DISPLAY_OK_HOSTS=false).
        type: bool
        default: False
        env:
          - name: PROFILE_ROLES_SUMMARY_ONLY
        ini:
          - section: callback_profile_roles
            key: summary_only
        version_added: 1.5.0
'''

import collections
import time

from ansible.plugins.callback import CallbackBase
from functools import reduce

# define start time
t0 = tn = time.time()


def secondsToStr(t):
    # http://bytes.com/topic/python/answers/635958-handy-short-cut-formatting-elapsed-time-floating-point-seconds
    def rediv(ll, b):
        return list(divmod(ll[0], b)) + ll[1:]

    return "%d:%02d:%02d.%03d" % tuple(
        reduce(rediv, [[t * 1000, ], 1000, 60, 60]))


def filled(msg, fchar="*"):
    if len(msg) == 0:
        width = 79
    else:
        msg = "%s " % msg
        width = 79 - len(msg)
    if width < 3:
        width = 3
    filler = fchar * width
    return "%s%s " % (msg, filler)


def timestamp(self):
    if self.current is not None:
        self.stats[self.current] = time.time() - self.stats[self.current]
        self.totals[self.current] += self.stats[self.current]


def tasktime():
    global tn
    time_current = time.strftime('%A %d %B %Y  %H:%M:%S %z')
    time_elapsed = secondsToStr(time.time() - tn)
    time_total_elapsed = secondsToStr(time.time() - t0)
    tn = time.time()
    return filled('%s (%s)%s%s' %
                  (time_current, time_elapsed, ' ' * 7, time_total_elapsed))


class CallbackModule(CallbackBase):
    """
    This callback module provides profiling for ansible roles.
    """
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'aggregate'
    CALLBACK_NAME = 'ansible.posix.profile_roles'
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self):
        self.stats = collections.Counter()
        self.totals = collections.Counter()
        self.current = None

        self.summary_only = None

        super(CallbackModule, self).__init__()

    def set_options(self, task_keys=None, var_options=None, direct=None):

        super(CallbackModule, self).set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.summary_only = self.get_option('summary_only')

    def _display_tasktime(self):
        if not self.summary_only:
            self._display.display(tasktime())

    def _record_task(self, task):
        """
        Logs the start of each task
        """
        self._display_tasktime()
        timestamp(self)

        if task._role:
            self.current = task._role._role_name
        else:
            self.current = task.action

        self.stats[self.current] = time.time()

    def v2_playbook_on_task_start(self, task, is_conditional):
        self._record_task(task)

    def v2_playbook_on_handler_task_start(self, task):
        self._record_task(task)

    def v2_playbook_on_stats(self, stats):
        # Align summary report header with other callback plugin summary
        self._display.banner("ROLES RECAP")

        self._display.display(tasktime())
        self._display.display(filled("", fchar="="))

        timestamp(self)
        total_time = sum(self.totals.values())

        # Print the timings starting with the largest one
        for result in self.totals.most_common():
            msg = u"{0:-<70}{1:->9}".format(result[0] + u' ', u' {0:.02f}s'.format(result[1]))
            self._display.display(msg)

        msg_total = u"{0:-<70}{1:->9}".format(u'total ', u' {0:.02f}s'.format(total_time))
        self._display.display(filled("", fchar="~"))
        self._display.display(msg_total)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2023 Ansible Project
# Copyright (c) 2023 Gomathi Selvi Srinivasan (@GomathiselviS)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: rds_global_cluster_info
version_added: 7.0.0
short_description: Obtain information about Aurora global database clusters
description:
  - Obtain information about Aurora global database clusters.
options:
    global_cluster_identifier:
        description:
          - The user-supplied Global DB cluster identifier.
          - If this parameter is specified, information from only the specific DB cluster is returned.
          - This parameter is not case-sensitive.
          - If supplied, must match an existing DBClusterIdentifier.
        type: str

author:
  - Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
  - While developing this module, describe_global_cluster CLI did not yield any tag information.
  - Consequently, the "tags" parameter is not included in this module.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get info of all existing DB clusters
  amazon.aws.rds_global_cluster_info:
  register: _result_cluster_info

- name: Get info on a specific DB cluster
  amazon.aws.rds_global_cluster_info:
    global_cluster_identifier: "{{ cluster_id }}"
  register: _result_global_cluster_info
"""

RETURN = r"""
global_clusters:
  description: List of global clusters.
  returned: always
  type: list
  elements: dict
  contains:
    global_cluster_identifier:
        description: User-supplied global database cluster identifier.
        type: str
        sample: "ansible-test-global-cluster"
    global_cluster_resource_id:
        description:
        - The Amazon Web Services Region-unique, immutable identifier for the global database cluster.
        type: str
        returned: always
        sample: "cluster-123456789012"
    global_cluster_arn:
        description:
        - The Amazon Resource Name (ARN) for the global database cluster.
        type: str
        returned: always
        sample: "arn:aws:rds::123456789012:global-cluster:ansible-test-global-cluster"
    status:
        description: The status of the DB cluster.
        type: str
        returned: always
        sample: "available"
    engine:
        description: The database engine of the DB cluster.
        type: str
        returned: always
        sample: "aurora-postgresql"
    engine_version:
        description: The database engine version.
        type: str
        returned: always
        sample: "14.8"
    storage_encrypted:
        description: Whether the DB cluster is storage encrypted.
        type: bool
        returned: always
        sample: false
    deletion_protection:
        description:
        - Indicates if the DB cluster has deletion protection enabled.
          The database can't be deleted when deletion protection is enabled.
        type: bool
        returned: always
        sample: false
    gloabl_cluster_members:
        description:
        - The list of primary and secondary clusters within the global database
          cluster.
        type: list
        elements: dict
        contains:
            db_cluster_arn:
                description: The Amazon Resource Name (ARN) for each Aurora DB cluster in the global cluster.
                type: str
                returned: always
                sample: "arn:aws:rds:us-east-1:123456789012:cluster:ansible-test-primary"
            readers:
                description: The Amazon Resource Name (ARN) for each read-only secondary cluster associated with the global cluster.
                type: list
                elements: str
                returned: always
                sample: "arn:aws:rds:us-east-2:123456789012:cluster:ansible-test-secondary"
            is_writer:
                description:
                - Indicates whether the Aurora DB cluster is the primary cluster for the global cluster with which it is associated.
                returned: always
                type: bool
                sample: false
            global_write_forwarding_status:
                description: The status of write forwarding for a secondary cluster in the global cluster.
                type: str
                returned: always
                sample: disabled
    failover_state:
        description:
        - A data object containing all properties for the current state of an in-process or
          pending switchover or failover process for this global cluster (Aurora global database).
        - This object is empty unless the SwitchoverGlobalCluster or FailoverGlobalCluster operation was called on this global cluster.
        type: dict
        contains:
            status:
                description:
                - The current status of the global cluster.
                type: str
                returned: always
                sample: "pending"
            from_db_cluster_arn:
                description: The Amazon Resource Name (ARN) of the Aurora DB cluster that is currently being demoted, and which is associated with this state.
                type: str
                returned: always
                sample: "arn:aws:rds:us-east-1:123456789012:cluster:ansible-test-primary"
            to_db_cluster_arn:
                description: The Amazon Resource Name (ARN) of the Aurora DB cluster that is currently being promoted, and which is associated with this state.
                type: str
                returned: always
                sample: "arn:aws:rds:us-east-2:123456789012:cluster:ansible-test-secondary"
            is_data_loss_allowed:
                description:
                - Indicates whether the operation is a global switchover or a global failover.
                - If data loss is allowed, then the operation is a global failover. Otherwise, it is a switchover.
                type: bool
                returned: always
                sample: false
"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


@AWSRetry.jittered_backoff(retries=10)
def _describe_global_clusters(client, **params):
    try:
        paginator = client.get_paginator("describe_global_clusters")
        return paginator.paginate(**params).build_full_result()["GlobalClusters"]
    except is_boto3_error_code("GlobalClusterNotFoundFault"):
        return []


def cluster_info(client, module):
    global_cluster_id = module.params.get("global_cluster_identifier")

    params = dict()
    if global_cluster_id:
        params["GlobalClusterIdentifier"] = global_cluster_id

    try:
        result = _describe_global_clusters(client, **params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, "Couldn't get Global cluster information.")

    return dict(
        changed=False, global_clusters=[camel_dict_to_snake_dict(cluster, ignore_list=["Tags"]) for cluster in result]
    )


def main():
    argument_spec = dict(
        global_cluster_identifier=dict(),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        client = module.client("rds", retry_decorator=AWSRetry.jittered_backoff(retries=10))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    module.exit_json(**cluster_info(client, module))


if __name__ == "__main__":
    main()



# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: splunk
type: notification
short_description: Sends task result events to Splunk HTTP Event Collector
author: "Stuart Hirst (!UNKNOWN) <support@convergingdata.com>"
description:
  - This callback plugin sends task results as JSON formatted events to a Splunk HTTP collector.
  - The companion Splunk Monitoring & Diagnostics App is available here U(https://splunkbase.splunk.com/app/4023/).
  - Credit to "Ryan Currah (@ryancurrah)" for original source upon which this is based.
requirements:
  - Whitelisting this callback plugin
  - 'Create a HTTP Event Collector in Splunk'
  - 'Define the URL and token in C(ansible.cfg)'
options:
  url:
    description: URL to the Splunk HTTP collector source.
    type: str
    env:
      - name: SPLUNK_URL
    ini:
      - section: callback_splunk
        key: url
  authtoken:
    description: Token to authenticate the connection to the Splunk HTTP collector.
    type: str
    env:
      - name: SPLUNK_AUTHTOKEN
    ini:
      - section: callback_splunk
        key: authtoken
  validate_certs:
    description: Whether to validate certificates for connections to HEC. It is not recommended to set to V(false) except
      when you are sure that nobody can intercept the connection between this plugin and HEC, as setting it to V(false) allows
      man-in-the-middle attacks!
    env:
      - name: SPLUNK_VALIDATE_CERTS
    ini:
      - section: callback_splunk
        key: validate_certs
    type: bool
    default: true
    version_added: '1.0.0'
  include_milliseconds:
    description: Whether to include milliseconds as part of the generated timestamp field in the event sent to the Splunk
      HTTP collector.
    env:
      - name: SPLUNK_INCLUDE_MILLISECONDS
    ini:
      - section: callback_splunk
        key: include_milliseconds
    type: bool
    default: false
    version_added: 2.0.0
  batch:
    description:
      - Correlation ID which can be set across multiple playbook executions.
    env:
      - name: SPLUNK_BATCH
    ini:
      - section: callback_splunk
        key: batch
    type: str
    version_added: 3.3.0
"""

EXAMPLES = r"""
examples: >-
  To enable, add this to your ansible.cfg file in the defaults block
    [defaults]
    callback_whitelist = community.general.splunk
  Set the environment variable
    export SPLUNK_URL=http://mysplunkinstance.datapaas.io:8088/services/collector/event
    export SPLUNK_AUTHTOKEN=f23blad6-5965-4537-bf69-5b5a545blabla88
  Set the ansible.cfg variable in the callback_splunk block
    [callback_splunk]
    url = http://mysplunkinstance.datapaas.io:8088/services/collector/event
    authtoken = f23blad6-5965-4537-bf69-5b5a545blabla88
"""

import getpass
import json
import socket
import uuid
from os.path import basename

from ansible.module_utils.ansible_release import __version__ as ansible_version
from ansible.module_utils.urls import open_url
from ansible.parsing.ajson import AnsibleJSONEncoder
from ansible.plugins.callback import CallbackBase

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


class SplunkHTTPCollectorSource:
    def __init__(self):
        self.ansible_check_mode = False
        self.ansible_playbook = ""
        self.session = str(uuid.uuid4())
        self.host = socket.gethostname()
        self.ip_address = socket.gethostbyname(socket.gethostname())
        self.user = getpass.getuser()

    def send_event(self, url, authtoken, validate_certs, include_milliseconds, batch, state, result, runtime):
        if result._task_fields["args"].get("_ansible_check_mode") is True:
            self.ansible_check_mode = True

        if result._task._role:
            ansible_role = str(result._task._role)
        else:
            ansible_role = None

        if "args" in result._task_fields:
            del result._task_fields["args"]

        data = {}
        data["uuid"] = result._task._uuid
        data["session"] = self.session
        if batch is not None:
            data["batch"] = batch
        data["status"] = state

        if include_milliseconds:
            time_format = "%Y-%m-%d %H:%M:%S.%f +0000"
        else:
            time_format = "%Y-%m-%d %H:%M:%S +0000"

        data["timestamp"] = now().strftime(time_format)
        data["host"] = self.host
        data["ip_address"] = self.ip_address
        data["user"] = self.user
        data["runtime"] = runtime
        data["ansible_version"] = ansible_version
        data["ansible_check_mode"] = self.ansible_check_mode
        data["ansible_host"] = result._host.name
        data["ansible_playbook"] = self.ansible_playbook
        data["ansible_role"] = ansible_role
        data["ansible_task"] = result._task_fields
        data["ansible_result"] = result._result

        # This wraps the json payload in and outer json event needed by Splunk
        jsondata = json.dumps({"event": data}, cls=AnsibleJSONEncoder, sort_keys=True)

        open_url(
            url,
            jsondata,
            headers={"Content-type": "application/json", "Authorization": f"Splunk {authtoken}"},
            method="POST",
            validate_certs=validate_certs,
        )


class CallbackModule(CallbackBase):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "community.general.splunk"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self, display=None):
        super().__init__(display=display)
        self.start_datetimes = {}  # Collect task start times
        self.url = None
        self.authtoken = None
        self.validate_certs = None
        self.include_milliseconds = None
        self.batch = None
        self.splunk = SplunkHTTPCollectorSource()

    def _runtime(self, result):
        return (now() - self.start_datetimes[result._task._uuid]).total_seconds()

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super().set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.url = self.get_option("url")

        if self.url is None:
            self.disabled = True
            self._display.warning(
                "Splunk HTTP collector source URL was "
                "not provided. The Splunk HTTP collector "
                "source URL can be provided using the "
                "`SPLUNK_URL` environment variable or "
                "in the ansible.cfg file."
            )

        self.authtoken = self.get_option("authtoken")

        if self.authtoken is None:
            self.disabled = True
            self._display.warning(
                "Splunk HTTP collector requires an authentication"
                "token. The Splunk HTTP collector "
                "authentication token can be provided using the "
                "`SPLUNK_AUTHTOKEN` environment variable or "
                "in the ansible.cfg file."
            )

        self.validate_certs = self.get_option("validate_certs")

        self.include_milliseconds = self.get_option("include_milliseconds")

        self.batch = self.get_option("batch")

    def v2_playbook_on_start(self, playbook):
        self.splunk.ansible_playbook = basename(playbook._file_name)

    def v2_playbook_on_task_start(self, task, is_conditional):
        self.start_datetimes[task._uuid] = now()

    def v2_playbook_on_handler_task_start(self, task):
        self.start_datetimes[task._uuid] = now()

    def v2_runner_on_ok(self, result, **kwargs):
        self.splunk.send_event(
            self.url,
            self.authtoken,
            self.validate_certs,
            self.include_milliseconds,
            self.batch,
            "OK",
            result,
            self._runtime(result),
        )

    def v2_runner_on_skipped(self, result, **kwargs):
        self.splunk.send_event(
            self.url,
            self.authtoken,
            self.validate_certs,
            self.include_milliseconds,
            self.batch,
            "SKIPPED",
            result,
            self._runtime(result),
        )

    def v2_runner_on_failed(self, result, **kwargs):
        self.splunk.send_event(
            self.url,
            self.authtoken,
            self.validate_certs,
            self.include_milliseconds,
            self.batch,
            "FAILED",
            result,
            self._runtime(result),
        )

    def v2_runner_on_async_failed(self, result, **kwargs):
        self.splunk.send_event(
            self.url,
            self.authtoken,
            self.validate_certs,
            self.include_milliseconds,
            self.batch,
            "FAILED",
            result,
            self._runtime(result),
        )

    def v2_runner_on_unreachable(self, result, **kwargs):
        self.splunk.send_event(
            self.url,
            self.authtoken,
            self.validate_certs,
            self.include_milliseconds,
            self.batch,
            "UNREACHABLE",
            result,
            self._runtime(result),
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Cindy Zhao (@cizhao) <cizhao@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_site_anp_epg_selector
short_description: Manage site-local EPG selector in schema templates
description:
- Manage EPG selector in schema template on Cisco ACI Multi-Site.
author:
- Cindy Zhao (@cizhao)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  site:
    description:
    - The name of the site.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  anp:
    description:
    - The name of the ANP.
    type: str
    required: true
  epg:
    description:
    - The name of the EPG to manage.
    type: str
    required: true
  selector:
    description:
    - The name of the selector.
    type: str
  expressions:
    description:
    - Expressions associated to this selector.
    type: list
    elements: dict
    suboptions:
      type:
        description:
        - The type of the expression.
        - The type is custom or is one of region, zone and ip_address
        - The type can be zone only when the site is AWS.
        required: true
        type: str
        aliases: [ tag ]
      operator:
        description:
        - The operator associated to the expression.
        - Operator has_key or does_not_have_key is only available for custom type / tag
        required: true
        type: str
        choices: [ not_in, in, equals, not_equals, has_key, does_not_have_key ]
      value:
        description:
        - The value associated to the expression.
        - If the operator is in or not_in, the value should be a comma separated string.
        type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_site_anp_epg
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a selector to a site EPG
  cisco.mso.mso_schema_site_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    site: Site 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    selector: selector_1
    expressions:
      - type: expression_1
        operator: in
        value: test
    state: present

- name: Remove a Selector from a site EPG
  cisco.mso.mso_schema_site_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    site: Site 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    selector: selector_1
    state: absent

- name: Query a specific Selector
  cisco.mso.mso_schema_site_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    site: Site 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    selector: selector_1
    state: query
  register: query_result

- name: Query all Selectors
  cisco.mso.mso_schema_site_anp_epg_selector:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    site: Site 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_expression_spec

EXPRESSION_KEYS = {
    "ip_address": "ipAddress",
    "region": "region",
    "zone": "zone",
}

EXPRESSION_OPERATORS = {
    "not_in": "notIn",
    "not_equals": "notEquals",
    "has_key": "keyExist",
    "does_not_have_key": "keyNotExist",
    "in": "in",
    "equals": "equals",
}


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        site=dict(type="str", required=True),
        template=dict(type="str", required=True),
        anp=dict(type="str", required=True),
        epg=dict(type="str", required=True),
        selector=dict(type="str"),
        expressions=dict(type="list", elements="dict", options=mso_expression_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["selector"]],
            ["state", "present", ["selector"]],
        ],
    )

    schema = module.params.get("schema")
    site = module.params.get("site")
    template = module.params.get("template").replace(" ", "")
    anp = module.params.get("anp")
    epg = module.params.get("epg")
    selector = module.params.get("selector")
    expressions = module.params.get("expressions")
    state = module.params.get("state")

    mso = MSOModule(module)

    # Get schema objects
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get template
    templates = [t.get("name") for t in schema_obj.get("templates")]
    if template not in templates:
        mso.fail_json(msg="Provided template '{0}' does not exist. Existing templates: {1}".format(template, ", ".join(templates)))
    template_idx = templates.index(template)

    # Get site
    site_id = mso.lookup_site(site)

    # Get cloud type
    site_type = mso.get_obj("sites", name=site).get("cloudProviders")[0]

    # Get site_idx
    if not schema_obj.get("sites"):
        mso.fail_json(msg="No site associated with template '{0}'. Associate the site with the template using mso_schema_site.".format(template))
    sites = [(s.get("siteId"), s.get("templateName")) for s in schema_obj.get("sites")]
    if (site_id, template) not in sites:
        mso.fail_json(msg="Provided site-template association '{0}-{1}' does not exist.".format(site, template))

    # Schema-access uses indexes
    site_idx = sites.index((site_id, template))
    # Path-based access uses site_id-template
    site_template = "{0}-{1}".format(site_id, template)

    payload = dict()
    ops = []
    op_path = ""
    selector_path = None

    # Get ANP
    anp_ref = mso.anp_ref(schema_id=schema_id, template=template, anp=anp)
    anps = [a.get("anpRef") for a in schema_obj["sites"][site_idx]["anps"]]
    anps_in_temp = [a.get("name") for a in schema_obj["templates"][template_idx]["anps"]]
    if anp not in anps_in_temp:
        mso.fail_json(msg="Provided anp '{0}' does not exist. Existing anps: {1}".format(anp, ", ".join(anps_in_temp)))
    else:
        # Get anp index at template level
        template_anp_idx = anps_in_temp.index(anp)

    # If anp not at site level but exists at template level
    if anp_ref not in anps:
        op_path = "/sites/{0}/anps/-".format(site_template)
        payload.update(
            anpRef=dict(
                schemaId=schema_id,
                templateName=template,
                anpName=anp,
            ),
        )

    else:
        # Get anp index at site level
        anp_idx = anps.index(anp_ref)

    # Get EPG
    epg_ref = mso.epg_ref(schema_id=schema_id, template=template, anp=anp, epg=epg)

    # If anp exists at site level
    if "anpRef" not in payload:
        epgs = [e.get("epgRef") for e in schema_obj["sites"][site_idx]["anps"][anp_idx]["epgs"]]

    # If anp already at site level AND if epg not at site level (or) anp not at site level?
    if ("anpRef" not in payload and epg_ref not in epgs) or "anpRef" in payload:
        epgs_in_temp = [e.get("name") for e in schema_obj["templates"][template_idx]["anps"][template_anp_idx]["epgs"]]

        # If EPG not at template level - Fail
        if epg not in epgs_in_temp:
            mso.fail_json(msg="Provided EPG '{0}' does not exist. Existing EPGs: {1}".format(epg, ", ".join(epgs_in_temp)))

        # EPG at template level but not at site level. Create payload at site level for EPG
        else:
            new_epg = dict(
                epgRef=dict(
                    schemaId=schema_id,
                    templateName=template,
                    anpName=anp,
                    epgName=epg,
                )
            )

            # If anp not in payload then, anp already exists at site level. New payload will only have new EPG payload
            if "anpRef" not in payload:
                op_path = "/sites/{0}/anps/{1}/epgs/-".format(site_template, anp)
                payload = new_epg
            else:
                # If anp in payload, anp exists at site level. Update payload with EPG payload
                payload["epgs"] = [new_epg]

    # Get index of EPG at site level
    else:
        epg_idx = epgs.index(epg_ref)

    # Get selectors
    # If anp at site level and epg is at site level
    if "anpRef" not in payload and "epgRef" not in payload:
        if selector and " " in selector:
            mso.fail_json(msg="There should not be any space in selector name.")
        selectors = [s.get("name") for s in schema_obj.get("sites")[site_idx]["anps"][anp_idx]["epgs"][epg_idx]["selectors"]]
        if selector in selectors:
            selector_idx = selectors.index(selector)
            selector_path = "/sites/{0}/anps/{1}/epgs/{2}/selectors/{3}".format(site_template, anp, epg, selector_idx)
            mso.existing = schema_obj["sites"][site_idx]["anps"][anp_idx]["epgs"][epg_idx]["selectors"][selector_idx]

    if state == "query":
        if "anpRef" in payload:
            mso.fail_json(msg="Anp '{anp}' does not exist in site level.".format(anp=anp))
        if "epgRef" in payload:
            mso.fail_json(msg="Epg '{epg}' does not exist in site level.".format(epg=epg))
        if selector is None:
            mso.existing = schema_obj["sites"][site_idx]["anps"][anp_idx]["epgs"][epg_idx]["selectors"]
        elif not mso.existing:
            mso.fail_json(msg="Selector '{selector}' not found".format(selector=selector))
        mso.exit_json()

    mso.previous = mso.existing
    if state == "absent":
        if mso.existing and selector_path:
            mso.sent = mso.existing = {}
            ops.append(dict(op="remove", path=selector_path))
    elif state == "present":
        # Get expressions
        all_expressions = []
        if expressions:
            for expression in expressions:
                type = expression.get("type")
                operator = expression.get("operator")
                value = expression.get("value")
                if " " in type:
                    mso.fail_json(msg="There should not be any space in 'type' attribute of expression '{0}'".format(type))
                if operator in ["has_key", "does_not_have_key"] and value:
                    mso.fail_json(msg="Attribute 'value' is not supported for operator '{0}' in expression '{1}'".format(operator, type))
                if operator in ["not_in", "in", "equals", "not_equals"] and not value:
                    mso.fail_json(msg="Attribute 'value' needed for operator '{0}' in expression '{1}'".format(operator, type))
                if type in ["region", "zone", "ip_address"]:
                    if type == "zone" and site_type != "aws":
                        mso.fail_json(msg="Type 'zone' is only supported for aws")
                    if operator in ["has_key", "does_not_have_key"]:
                        mso.fail_json(msg="Operator '{0}' is not supported when expression type is '{1}'".format(operator, type))
                    type = EXPRESSION_KEYS.get(type)
                else:
                    type = "Custom:" + type
                all_expressions.append(
                    dict(
                        key=type,
                        operator=EXPRESSION_OPERATORS.get(operator),
                        value=value,
                    )
                )
        new_selector = dict(
            name=selector,
            expressions=all_expressions,
        )

        selectors_path = "/sites/{0}/anps/{1}/epgs/{2}/selectors/-".format(site_template, anp, epg)

        # if payload is empty, anp and epg already exist at site level
        if not payload:
            op_path = selectors_path
            payload = new_selector
        # if payload exist
        else:
            # if anp already exists at site level
            if "anpRef" not in payload:
                payload["selectors"] = [new_selector]
            else:
                payload["epgs"][0]["selectors"] = [new_selector]

        mso.sanitize(payload, collate=True)

        if mso.existing and selector_path:
            ops.append(dict(op="replace", path=selector_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=op_path, value=mso.sent))

        mso.existing = new_selector

    if not module.check_mode and mso.existing != mso.previous:
        mso.request(schema_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: manageiq_alert_profiles

short_description: Configuration of alert profiles for ManageIQ
extends_documentation_fragment:
  - community.general._manageiq
  - community.general._attributes

author: Elad Alfassa (@elad661) <ealfassa@redhat.com>
description:
  - The manageiq_alert_profiles module supports adding, updating and deleting alert profiles in ManageIQ.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  state:
    type: str
    description:
      - V(absent) - alert profile should not exist,
      - V(present) - alert profile should exist.
    choices: ['absent', 'present']
    default: 'present'
  name:
    type: str
    description:
      - The unique alert profile name in ManageIQ.
    required: true
  resource_type:
    type: str
    description:
      - The resource type for the alert profile in ManageIQ. Required when O(state=present).
    choices: ['Vm', 'ContainerNode', 'MiqServer', 'Host', 'Storage', 'EmsCluster', 'ExtManagementSystem', 'MiddlewareServer']
  alerts:
    type: list
    elements: str
    description:
      - List of alert descriptions to assign to this profile.
      - Required if O(state=present).
  notes:
    type: str
    description:
      - Optional notes for this profile.
"""

EXAMPLES = r"""
- name: Add an alert profile to ManageIQ
  community.general.manageiq_alert_profiles:
    state: present
    name: Test profile
    resource_type: ContainerNode
    alerts:
      - Test Alert 01
      - Test Alert 02
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!

- name: Delete an alert profile from ManageIQ
  community.general.manageiq_alert_profiles:
    state: absent
    name: Test profile
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._manageiq import ManageIQ, manageiq_argument_spec


class ManageIQAlertProfiles:
    """Object to execute alert profile management operations in manageiq."""

    def __init__(self, manageiq):
        self.manageiq = manageiq

        self.module = self.manageiq.module
        self.api_url = self.manageiq.api_url
        self.client = self.manageiq.client
        self.url = f"{self.api_url}/alert_definition_profiles"

    def get_profiles(self):
        """Get all alert profiles from ManageIQ"""
        try:
            response = self.client.get(f"{self.url}?expand=alert_definitions,resources")
        except Exception as e:
            self.module.fail_json(msg=f"Failed to query alert profiles: {e}")
        return response.get("resources") or []

    def get_alerts(self, alert_descriptions):
        """Get a list of alert hrefs from a list of alert descriptions"""
        alerts = []
        for alert_description in alert_descriptions:
            alert = self.manageiq.find_collection_resource_or_fail("alert_definitions", description=alert_description)
            alerts.append(alert["href"])

        return alerts

    def add_profile(self, profile):
        """Add a new alert profile to ManageIQ"""
        # find all alerts to add to the profile
        # we do this first to fail early if one is missing.
        alerts = self.get_alerts(profile["alerts"])

        # build the profile dict to send to the server

        profile_dict = dict(name=profile["name"], description=profile["name"], mode=profile["resource_type"])
        if profile["notes"]:
            profile_dict["set_data"] = dict(notes=profile["notes"])

        # send it to the server
        try:
            result = self.client.post(self.url, resource=profile_dict, action="create")
        except Exception as e:
            self.module.fail_json(msg=f"Creating profile failed {e}")

        # now that it has been created, we can assign the alerts
        self.assign_or_unassign(result["results"][0], alerts, "assign")

        msg = f"Profile {profile['name']} created successfully"
        return dict(changed=True, msg=msg)

    def delete_profile(self, profile):
        """Delete an alert profile from ManageIQ"""
        try:
            self.client.post(profile["href"], action="delete")
        except Exception as e:
            self.module.fail_json(msg=f"Deleting profile failed: {e}")

        msg = f"Successfully deleted profile {profile['name']}"
        return dict(changed=True, msg=msg)

    def get_alert_href(self, alert):
        """Get an absolute href for an alert"""
        return f"{self.api_url}/alert_definitions/{alert['id']}"

    def assign_or_unassign(self, profile, resources, action):
        """Assign or unassign alerts to profile, and validate the result."""
        alerts = [dict(href=href) for href in resources]

        subcollection_url = f"{profile['href']}/alert_definitions"
        try:
            result = self.client.post(subcollection_url, resources=alerts, action=action)
            if len(result["results"]) != len(alerts):
                msg = (
                    f"Failed to {action} alerts to profile '{profile['name']}', "
                    f"expected {len(alerts)} alerts to be {action}ed, "
                    f"but only {result['results']} were {action}ed"
                )
                self.module.fail_json(msg=msg)
        except Exception as e:
            msg = f"Failed to {action} alerts to profile '{profile['name']}': {e}"
            self.module.fail_json(msg=msg)

        return result["results"]

    def update_profile(self, old_profile, desired_profile):
        """Update alert profile in ManageIQ"""
        changed = False
        # we need to use client.get to query the alert definitions
        old_profile = self.client.get(f"{old_profile['href']}?expand=alert_definitions")

        # figure out which alerts we need to assign / unassign
        # alerts listed by the user:
        desired_alerts = set(self.get_alerts(desired_profile["alerts"]))

        # alert which currently exist in the profile
        if "alert_definitions" in old_profile:
            # we use get_alert_href to have a direct href to the alert
            existing_alerts = {self.get_alert_href(alert) for alert in old_profile["alert_definitions"]}
        else:
            # no alerts in this profile
            existing_alerts = set()

        to_add = list(desired_alerts - existing_alerts)
        to_remove = list(existing_alerts - desired_alerts)

        # assign / unassign the alerts, if needed

        if to_remove:
            self.assign_or_unassign(old_profile, to_remove, "unassign")
            changed = True
        if to_add:
            self.assign_or_unassign(old_profile, to_add, "assign")
            changed = True

        # update other properties
        profile_dict = dict()

        if old_profile["mode"] != desired_profile["resource_type"]:
            # mode needs to be updated
            profile_dict["mode"] = desired_profile["resource_type"]

        # check if notes need to be updated
        old_notes = old_profile.get("set_data", {}).get("notes")

        if desired_profile["notes"] != old_notes:
            profile_dict["set_data"] = dict(notes=desired_profile["notes"])

        if profile_dict:
            # if we have any updated values
            changed = True
            try:
                self.client.post(old_profile["href"], resource=profile_dict, action="edit")
            except Exception as e:
                msg = f"Updating profile '{old_profile['name']}' failed: {e}"
                self.module.fail_json(msg=msg)

        if changed:
            msg = f"Profile {desired_profile['name']} updated successfully"
        else:
            msg = f"No update needed for profile {desired_profile['name']}"
        return dict(changed=changed, msg=msg)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        resource_type=dict(
            type="str",
            choices=[
                "Vm",
                "ContainerNode",
                "MiqServer",
                "Host",
                "Storage",
                "EmsCluster",
                "ExtManagementSystem",
                "MiddlewareServer",
            ],
        ),
        alerts=dict(type="list", elements="str"),
        notes=dict(type="str"),
        state=dict(default="present", choices=["present", "absent"]),
    )
    # add the manageiq connection arguments to the arguments
    argument_spec.update(manageiq_argument_spec())

    module = AnsibleModule(argument_spec=argument_spec, required_if=[("state", "present", ["resource_type", "alerts"])])

    state = module.params["state"]
    name = module.params["name"]

    manageiq = ManageIQ(module)
    manageiq_alert_profiles = ManageIQAlertProfiles(manageiq)

    existing_profile = manageiq.find_collection_resource_by("alert_definition_profiles", name=name)

    # we need to add or update the alert profile
    if state == "present":
        if not existing_profile:
            # a profile with this name doesn't exist yet, let's create it
            res_args = manageiq_alert_profiles.add_profile(module.params)
        else:
            # a profile with this name exists, we might need to update it
            res_args = manageiq_alert_profiles.update_profile(existing_profile, module.params)

    # this alert profile should not exist
    if state == "absent":
        # if we have an alert profile with this name, delete it
        if existing_profile:
            res_args = manageiq_alert_profiles.delete_profile(existing_profile)
        else:
            # This alert profile does not exist in ManageIQ, and that's okay
            msg = f"Alert profile '{name}' does not exist in ManageIQ"
            res_args = dict(changed=False, msg=msg)

    module.exit_json(**res_args)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: aws_az_info
short_description: Gather information about availability zones in AWS
version_added: 1.0.0
description:
    - Gather information about availability zones in AWS.
author: 'Henrique Rodrigues (@Sodki)'
options:
  filters:
    description:
      - A dict of filters to apply.
      - Each dict item consists of a filter key and a filter value.
      - See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeAvailabilityZones.html) for possible filters.
      - Filter names and values are case sensitive.
      - You can use underscores instead of dashes (-) in the filter keys.
      - Filter keys with underscores will take precedence in case of conflict.
    required: false
    default: {}
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all availability zones
  amazon.aws.aws_az_info:

- name: Gather information about a single availability zone
  amazon.aws.aws_az_info:
    filters:
      zone-name: eu-west-1a

- name: Gather information in a availability zones based on their state, such as "available"
  amazon.aws.aws_az_info:
    region: us-east-1
    filters:
      state: available
"""

RETURN = r"""
availability_zones:
    returned: on success
    description: >
        Availability zones that match the provided filters. Each element consists of a dict with all the information
        related to that available zone.
    type: list
    elements: dict
    contains:
        state:
            description:
                - The state of the availability zone.
                - The value is always C(available).
            type: str
            returned: on success
            sample: 'available'
        opt_in_status:
            description:
                - The opt-in status.
                - The value is always C(opt-in-not-required) for availability zones.
            type: str
            returned: on success
            sample: 'opt-in-not-required'
        messages:
            description: List of messages about the availability zone.
            type: list
            elements: dict
            contains:
                message:
                    description: The message about the availability zone.
                    type: str
                    returned: on success
                    sample: 'msg'
            returned: on success
            sample: [
                {
                    'message': 'message_one'
                },
                {
                    'message': 'message_two'
                }
            ]
        region_name:
            description: The name of the region.
            type: str
            returned: on success
            sample: 'us-east-1'
        zone_name:
            description: The name of the availability zone.
            type: str
            returned: on success
            sample: 'us-east-1e'
        zone_id:
            description: The ID of the availability zone.
            type: str
            returned: on success
            sample: 'use1-az5'
        group_name:
            description:
                - The name of the associated group.
                - For availability zones, this will be the same as RV(availability_zones.region_name).
            type: str
            returned: on success
            sample: 'us-east-1'
        network_border_group:
            description: The name of the network border group.
            type: str
            returned: on success
            sample: 'us-east-1'
        zone_type:
            description: The type of zone.
            type: str
            returned: on success
            sample: 'availability-zone'
    sample: [
        {
            "group_name": "us-east-1",
            "messages": [],
            "network_border_group": "us-east-1",
            "opt_in_status": "opt-in-not-required",
            "region_name": "us-east-1",
            "state": "available",
            "zone_id": "use1-az6",
            "zone_name": "us-east-1a",
            "zone_type": "availability-zone"
        },
        {
            "group_name": "us-east-1",
            "messages": [],
            "network_border_group": "us-east-1",
            "opt_in_status": "opt-in-not-required",
            "region_name": "us-east-1",
            "state": "available",
            "zone_id": "use1-az1",
            "zone_name": "us-east-1b",
            "zone_type": "availability-zone"
        }
    ]
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_availability_zones
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.transformation import sanitize_filters_to_boto3_filter_list


def main():
    argument_spec = dict(filters=dict(default={}, type="dict"))

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("ec2")

    # Sanitize filters
    sanitized_filters = sanitize_filters_to_boto3_filter_list(module.params.get("filters"))
    try:
        availability_zones = describe_availability_zones(connection, Filters=sanitized_filters)
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Unable to describe availability zones.")

    # Turn the boto3 result into ansible_friendly_snaked_names
    snaked_availability_zones = [camel_dict_to_snake_dict(az) for az in availability_zones]

    module.exit_json(availability_zones=snaked_availability_zones)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, 2019 Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
author:
  - Kevin Breit (@kbreit)
deprecated:
  alternative: cisco.meraki.networks
  removed_in: 3.0.0
  why: Updated modules released with increased functionality
description:
  - Allows for creation, management, and visibility into networks within Meraki.
extends_documentation_fragment: cisco.meraki.meraki
module: meraki_network
options:
  copy_from_network_id:
    description:
      - New network inherits properties from this network ID.
      - Other provided parameters will override the copied configuration.
      - Type which must match this network's type exactly.
    type: str
  enable_vlans:
    description:
      - Boolean value specifying whether VLANs should be supported on a network.
      - Requires C(net_name) or C(net_id) to be specified.
    type: bool
  local_status_page_enabled:
    description: '- This no longer works and will likely be moved to a separate module.
      - Enables the local device status pages (U[my.meraki.com](my.meraki.com), U[ap.meraki.com](ap.meraki.com),
      U[switch.meraki.com](switch.meraki.com), U[wired.meraki.com](wired.meraki.com)).
      - Only can be specified on its own or with C(remote_status_page_enabled). '
    type: bool
  net_id:
    description:
      - ID number of a network.
    type: str
  net_name:
    aliases:
      - name
      - network
    description:
      - Name of a network.
    type: str
  remote_status_page_enabled:
    description:
      - This no longer works and will likely be moved to a separate module.
      - Enables access to the device status page (U(http://device LAN IP)).
      - Can only be set if C(local_status_page_enabled:) is set to C(yes).
      - Only can be specified on its own or with C(local_status_page_enabled).
    type: bool
  state:
    choices:
      - absent
      - present
      - query
    default: present
    description:
      - Create or modify an organization.
    type: str
  tags:
    description:
      - List of tags to assign to network.
      - C(tags) name conflicts with the tags parameter in Ansible. Indentation problems
        may cause unexpected behaviors.
      - Ansible 2.8 converts this to a list from a comma separated list.
    elements: str
    type: list
  timezone:
    description:
      - Timezone associated to network.
      - See U(https://en.wikipedia.org/wiki/List_of_tz_database_time_zones) for a
        list of valid timezones.
    type: str
  type:
    aliases:
      - net_type
    choices:
      - appliance
      - switch
      - wireless
      - sensor
      - systemsManager
      - camera
      - cellularGateway
    description:
      - Type of network device network manages.
      - Required when creating a network.
      - As of Ansible 2.8, C(combined) type is no longer accepted.
      - As of Ansible 2.8, changes to this parameter are no longer idempotent.
    elements: str
    type: list
short_description: Manage networks in the Meraki cloud
"""

EXAMPLES = r"""
- delegate_to: localhost
  block:
    - name: List all networks associated to the YourOrg organization
      meraki_network:
        auth_key: abc12345
        state: query
        org_name: YourOrg
    - name: Query network named MyNet in the YourOrg organization
      meraki_network:
        auth_key: abc12345
        state: query
        org_name: YourOrg
        net_name: MyNet
    - name: Create network named MyNet in the YourOrg organization
      meraki_network:
        auth_key: abc12345
        state: present
        org_name: YourOrg
        net_name: MyNet
        type: switch
        timezone: America/Chicago
        tags: production, chicago
    - name: Create combined network named MyNet in the YourOrg organization
      meraki_network:
        auth_key: abc12345
        state: present
        org_name: YourOrg
        net_name: MyNet
        type:
          - switch
          - appliance
        timezone: America/Chicago
        tags: production, chicago
    - name: Create new network based on an existing network
      meraki_network:
        auth_key: abc12345
        state: present
        org_name: YourOrg
        net_name: MyNet
        type:
          - switch
          - appliance
        copy_from_network_id: N_1234
    - name: Enable VLANs on a network
      meraki_network:
        auth_key: abc12345
        state: query
        org_name: YourOrg
        net_name: MyNet
        enable_vlans: true
    - name: Modify local status page enabled state
      meraki_network:
        auth_key: abc12345
        state: query
        org_name: YourOrg
        net_name: MyNet
        local_status_page_enabled: true
"""

RETURN = r"""
data:
    description: Information about the created or manipulated object.
    returned: info
    type: complex
    contains:
      id:
        description: Identification string of network.
        returned: success
        type: str
        sample: N_12345
      name:
        description: Written name of network.
        returned: success
        type: str
        sample: YourNet
      organization_id:
        description: Organization ID which owns the network.
        returned: success
        type: str
        sample: 0987654321
      tags:
        description: Space delimited tags assigned to network.
        returned: success
        type: list
        sample: ['production']
      time_zone:
        description: Timezone where network resides.
        returned: success
        type: str
        sample: America/Chicago
      type:
        description: Functional type of network.
        returned: success
        type: list
        sample: ['switch']
      local_status_page_enabled:
        description: States whether U(my.meraki.com) and other device portals should be enabled.
        returned: success
        type: bool
        sample: true
      remote_status_page_enabled:
        description: Enables access to the device status page.
        returned: success
        type: bool
        sample: true
"""

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)


def is_net_valid(data, net_name=None, net_id=None):
    if net_name is None and net_id is None:
        return False
    for n in data:
        if net_name:
            if n["name"] == net_name:
                return True
        elif net_id:
            if n["id"] == net_id:
                return True
    return False


def get_network_settings(meraki, net_id):
    path = meraki.construct_path("get_settings", net_id=net_id)
    response = meraki.request(path, method="GET")
    return response


def main():

    # define the available arguments/parameters that a user can pass to
    # the module

    argument_spec = meraki_argument_spec()
    argument_spec.update(
        net_id=dict(type="str"),
        type=dict(
            type="list",
            elements="str",
            choices=[
                "wireless",
                "switch",
                "appliance",
                "sensor",
                "systemsManager",
                "camera",
                "cellularGateway",
            ],
            aliases=["net_type"],
        ),
        tags=dict(type="list", elements="str"),
        timezone=dict(type="str"),
        net_name=dict(type="str", aliases=["name", "network"]),
        state=dict(
            type="str", choices=["present", "query", "absent"], default="present"
        ),
        enable_vlans=dict(type="bool"),
        local_status_page_enabled=dict(type="bool"),
        remote_status_page_enabled=dict(type="bool"),
        copy_from_network_id=dict(type="str"),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    meraki = MerakiModule(module, function="network")
    module.params["follow_redirects"] = "all"
    payload = None

    create_urls = {"network": "/organizations/{org_id}/networks"}
    update_urls = {"network": "/networks/{net_id}"}
    delete_urls = {"network": "/networks/{net_id}"}
    update_settings_urls = {"network": "/networks/{net_id}/settings"}
    get_settings_urls = {"network": "/networks/{net_id}/settings"}
    enable_vlans_urls = {"network": "/networks/{net_id}/appliance/vlans/settings"}
    get_vlan_status_urls = {"network": "/networks/{net_id}/appliance/vlans/settings"}
    meraki.url_catalog["create"] = create_urls
    meraki.url_catalog["update"] = update_urls
    meraki.url_catalog["update_settings"] = update_settings_urls
    meraki.url_catalog["get_settings"] = get_settings_urls
    meraki.url_catalog["delete"] = delete_urls
    meraki.url_catalog["enable_vlans"] = enable_vlans_urls
    meraki.url_catalog["status_vlans"] = get_vlan_status_urls

    if not meraki.params["org_name"] and not meraki.params["org_id"]:
        meraki.fail_json(msg="org_name or org_id parameters are required")
    if meraki.params["state"] != "query":
        if not meraki.params["net_name"] and not meraki.params["net_id"]:
            meraki.fail_json(
                msg="net_name or net_id is required for present or absent states"
            )
    if meraki.params["net_name"] and meraki.params["net_id"]:
        meraki.fail_json(msg="net_name and net_id are mutually exclusive")
    if not meraki.params["net_name"] and not meraki.params["net_id"]:
        if meraki.params["enable_vlans"]:
            meraki.fail_json(
                msg="The parameter 'enable_vlans' requires 'net_name' or 'net_id' to be specified"
            )
    if (
        meraki.params["local_status_page_enabled"] is False
        and meraki.params["remote_status_page_enabled"] is True
    ):
        meraki.fail_json(
            msg="local_status_page_enabled must be true when setting remote_status_page_enabled"
        )

    # Construct payload
    if meraki.params["state"] == "present":
        payload = dict()
        if meraki.params["net_name"]:
            payload["name"] = meraki.params["net_name"]
        if meraki.params["type"]:
            payload["productTypes"] = meraki.params["type"]
        if meraki.params["tags"]:
            payload["tags"] = meraki.params["tags"]
        if meraki.params["timezone"]:
            payload["timeZone"] = meraki.params["timezone"]
        if meraki.params["local_status_page_enabled"] is not None:
            payload["localStatusPageEnabled"] = meraki.params[
                "local_status_page_enabled"
            ]
        if meraki.params["remote_status_page_enabled"] is not None:
            payload["remoteStatusPageEnabled"] = meraki.params[
                "remote_status_page_enabled"
            ]
        if meraki.params["copy_from_network_id"] is not None:
            payload["copyFromNetworkId"] = meraki.params["copy_from_network_id"]

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)

    org_id = meraki.params["org_id"]
    if not org_id:
        org_id = meraki.get_org_id(meraki.params["org_name"])
    nets = meraki.get_nets(org_id=org_id)
    net_id = meraki.params["net_id"]
    net_exists = False
    if net_id is not None:
        if is_net_valid(nets, net_id=net_id) is False:
            meraki.fail_json(msg="Network specified by net_id does not exist.")
        net_exists = True
    elif meraki.params["net_name"]:
        if is_net_valid(nets, net_name=meraki.params["net_name"]) is True:
            net_id = meraki.get_net_id(net_name=meraki.params["net_name"], data=nets)
            net_exists = True

    if meraki.params["state"] == "query":
        if not meraki.params["net_name"] and not meraki.params["net_id"]:
            meraki.result["data"] = nets
        elif meraki.params["net_name"] or meraki.params["net_id"] is not None:
            if (
                meraki.params["local_status_page_enabled"] is not None
                or meraki.params["remote_status_page_enabled"] is not None
            ):
                meraki.result["data"] = get_network_settings(meraki, net_id)
                meraki.exit_json(**meraki.result)
            else:
                meraki.result["data"] = meraki.get_net(
                    meraki.params["org_name"], net_name=meraki.params["net_name"], data=nets, net_id=meraki.params["net_id"],
                )
                meraki.exit_json(**meraki.result)
    elif meraki.params["state"] == "present":
        if net_exists is False:  # Network needs to be created
            if "type" not in meraki.params or meraki.params["type"] is None:
                meraki.fail_json(
                    msg="type parameter is required when creating a network."
                )
            if meraki.check_mode is True:
                data = payload
                data["id"] = "N_12345"
                data["organization_id"] = org_id
                meraki.result["data"] = data
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path("create", org_id=org_id)
            r = meraki.request(path, method="POST", payload=json.dumps(payload))
            if meraki.status == 201:
                meraki.result["data"] = r
                meraki.result["changed"] = True
        else:  # Network exists, make changes
            if meraki.params["enable_vlans"] is not None:  # Modify VLANs configuration
                status_path = meraki.construct_path("status_vlans", net_id=net_id)
                status = meraki.request(status_path, method="GET")
                payload = {"vlansEnabled": meraki.params["enable_vlans"]}
                if meraki.is_update_required(status, payload):
                    if meraki.check_mode is True:
                        data = {
                            "vlansEnabled": meraki.params["enable_vlans"],
                            "network_id": net_id,
                        }
                        meraki.result["data"] = data
                        meraki.result["changed"] = True
                        meraki.exit_json(**meraki.result)
                    path = meraki.construct_path("enable_vlans", net_id=net_id)
                    r = meraki.request(path, method="PUT", payload=json.dumps(payload))
                    if meraki.status == 200:
                        meraki.result["data"] = r
                        meraki.result["changed"] = True
                        meraki.exit_json(**meraki.result)
                else:
                    meraki.result["data"] = status
                    meraki.exit_json(**meraki.result)
            elif (
                meraki.params["local_status_page_enabled"] is not None
                or meraki.params["remote_status_page_enabled"] is not None
            ):
                path = meraki.construct_path("get_settings", net_id=net_id)
                original = meraki.request(path, method="GET")
                payload = {}
                if meraki.params["local_status_page_enabled"] is not None:
                    payload["localStatusPageEnabled"] = meraki.params[
                        "local_status_page_enabled"
                    ]
                if meraki.params["remote_status_page_enabled"] is not None:
                    payload["remoteStatusPageEnabled"] = meraki.params[
                        "remote_status_page_enabled"
                    ]
                if meraki.is_update_required(original, payload):
                    if meraki.check_mode is True:
                        original.update(payload)
                        meraki.result["data"] = original
                        meraki.result["changed"] = True
                        meraki.exit_json(**meraki.result)
                    path = meraki.construct_path("update_settings", net_id=net_id)
                    response = meraki.request(
                        path, method="PUT", payload=json.dumps(payload)
                    )
                    meraki.result["data"] = response
                    meraki.result["changed"] = True
                    meraki.exit_json(**meraki.result)
                else:
                    meraki.result["data"] = original
                    meraki.exit_json(**meraki.result)
            net = meraki.get_net(meraki.params["org_name"], net_id=net_id, data=nets)
            if meraki.is_update_required(net, payload):
                if meraki.check_mode is True:
                    data = net
                    net.update(payload)
                    meraki.result["data"] = net
                    meraki.result["changed"] = True
                    meraki.exit_json(**meraki.result)
                path = meraki.construct_path("update", net_id=net_id)
                r = meraki.request(path, method="PUT", payload=json.dumps(payload))
                if meraki.status == 200:
                    meraki.result["data"] = r
                    meraki.result["changed"] = True
            else:
                meraki.result["data"] = net
    elif meraki.params["state"] == "absent":
        if is_net_valid(nets, net_id=net_id) is True:
            if meraki.check_mode is True:
                meraki.result["data"] = {}
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path("delete", net_id=net_id)
            r = meraki.request(path, method="DELETE")
            if meraki.status == 204:
                meraki.result["changed"] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Guillaume Delpierre <gde@llew.me>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssl_pkcs12
author:
  - Guillaume Delpierre (@gdelpierre)
short_description: Generate OpenSSL PKCS#12 archive
description:
  - This module allows one to (re-)generate PKCS#12.
  - The module uses the cryptography Python library.
extends_documentation_fragment:
  - ansible.builtin.files
  - community.crypto._attributes
  - community.crypto._attributes.files
  - community.crypto._cryptography_dep.minimum
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  safe_file_operations:
    support: full
  idempotent:
    support: partial
    details:
      - The module is not idempotent if O(force=true).
options:
  action:
    description:
      - V(export) or V(parse) a PKCS#12.
    type: str
    default: export
    choices: [export, parse]
  other_certificates:
    description:
      - List of other certificates to include. Pre Ansible 2.8 this parameter was called O(ca_certificates).
      - Assumes there is one PEM-encoded certificate per file. If a file contains multiple PEM certificates, set O(other_certificates_parse_all)
        to V(true).
      - Mutually exclusive with O(other_certificates_content).
    type: list
    elements: path
    aliases: [ca_certificates]
  other_certificates_content:
    description:
      - List of other certificates to include.
      - Assumes there is one PEM-encoded certificate per item. If an item contains multiple PEM certificates, set O(other_certificates_parse_all)
      - Mutually exclusive with O(other_certificates).
    type: list
    elements: str
    version_added: "2.26.0"
  other_certificates_parse_all:
    description:
      - If set to V(true), assumes that the files mentioned in O(other_certificates)/O(other_certificates_content) can contain more than one
        certificate per file/item (or even none per file/item).
    type: bool
    default: false
    version_added: 1.4.0
  certificate_path:
    description:
      - The path to read certificates and private keys from.
      - Must be in PEM format.
      - Mutually exclusive with O(certificate_content).
    type: path
  certificate_content:
    description:
      - Content of the certificate file in PEM format.
      - Mutually exclusive with O(certificate_path).
    type: str
    version_added: "2.26.0"
  force:
    description:
      - Should the file be regenerated even if it already exists.
    type: bool
    default: false
  friendly_name:
    description:
      - Specifies the friendly name for the certificate and private key.
    type: str
    aliases: [name]
  iter_size:
    description:
      - Number of times to repeat the encryption step.
      - This is B(not considered during idempotency checks).
      - This is only used when O(encryption_level=compatibility2022).
      - When using it, the default is V(50000).
    type: int
  maciter_size:
    description:
      - Number of times to repeat the MAC step.
      - This is B(not considered during idempotency checks).
      - This value is B(not used). It is deprecated and will be removed from community.crypto 4.0.0.
    type: int
  encryption_level:
    description:
      - Determines the encryption level used.
      - V(auto) uses the default of the selected backend. For C(cryptography), this is what the cryptography library's specific
        version considers the best available encryption.
      - V(compatibility2022) uses compatibility settings for older software in 2022. This is only supported by the C(cryptography)
        backend if cryptography >= 38.0.0 is available.
      - B(Note) that this option is B(not used for idempotency).
    choices:
      - auto
      - compatibility2022
    default: auto
    type: str
    version_added: 2.8.0
  passphrase:
    description:
      - The PKCS#12 password.
      - B(Note:) PKCS12 encryption is typically not secure and should not be used as a security mechanism. If you need to
        store or send a PKCS12 file safely, you should additionally encrypt it with something else. (L(Source,
        https://cryptography.io/en/latest/hazmat/primitives/asymmetric/serialization/#cryptography.hazmat.primitives.serialization.pkcs12.serialize_key_and_certificates)).
    type: str
  path:
    description:
      - Filename to write the PKCS#12 file to.
    type: path
    required: true
  privatekey_passphrase:
    description:
      - Passphrase source to decrypt any input private keys with.
    type: str
  privatekey_path:
    description:
      - File to read private key from.
      - Mutually exclusive with O(privatekey_content).
    type: path
  privatekey_content:
    description:
      - Content of the private key file.
      - Mutually exclusive with O(privatekey_path).
    type: str
    version_added: "2.3.0"
  state:
    description:
      - Whether the file should exist or not. All parameters except O(path) are ignored when state is V(absent).
    choices: [absent, present]
    default: present
    type: str
  src:
    description:
      - PKCS#12 file path to parse.
    type: path
  backup:
    description:
      - Create a backup file including a timestamp so you can get the original output file back if you overwrote it with a
        new one by accident.
    type: bool
    default: false
  return_content:
    description:
      - If set to V(true), will return the (current or generated) PKCS#12's content as RV(pkcs12).
    type: bool
    default: false
    version_added: "1.0.0"
  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - The value V(pyopenssl) has been removed for community.crypto 3.0.0.
      - Note that with community.crypto 3.0.0, all remaining values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]
    version_added: 1.7.0
seealso:
  - module: community.crypto.x509_certificate
  - module: community.crypto.openssl_csr
  - module: community.crypto.openssl_dhparam
  - module: community.crypto.openssl_privatekey
  - module: community.crypto.openssl_publickey
"""

EXAMPLES = r"""
---
- name: Generate PKCS#12 file
  community.crypto.openssl_pkcs12:
    action: export
    path: /opt/certs/ansible.p12
    friendly_name: raclette
    privatekey_path: /opt/certs/keys/key.pem
    certificate_path: /opt/certs/cert.pem
    other_certificates: /opt/certs/ca.pem
  # Note that if /opt/certs/ca.pem contains multiple certificates,
  # only the first one will be used. See the other_certificates_parse_all
  # option for changing this behavior.
    state: present

- name: Generate PKCS#12 file
  community.crypto.openssl_pkcs12:
    action: export
    path: /opt/certs/ansible.p12
    friendly_name: raclette
    privatekey_content: '{{ private_key_contents }}'
    certificate_path: /opt/certs/cert.pem
    other_certificates_parse_all: true
    other_certificates:
      - /opt/certs/ca_bundle.pem
      # Since we set other_certificates_parse_all to true, all
      # certificates in the CA bundle are included and not just
      # the first one.
      - /opt/certs/intermediate.pem
      # In case this file has multiple certificates in it,
      # all will be included as well.
    state: present

- name: Change PKCS#12 file permission
  community.crypto.openssl_pkcs12:
    action: export
    path: /opt/certs/ansible.p12
    friendly_name: raclette
    privatekey_path: /opt/certs/keys/key.pem
    certificate_path: /opt/certs/cert.pem
    other_certificates: /opt/certs/ca.pem
    state: present
    mode: '0600'

- name: Regen PKCS#12 file
  community.crypto.openssl_pkcs12:
    action: export
    src: /opt/certs/ansible.p12
    path: /opt/certs/ansible.p12
    friendly_name: raclette
    privatekey_path: /opt/certs/keys/key.pem
    certificate_path: /opt/certs/cert.pem
    other_certificates: /opt/certs/ca.pem
    state: present
    mode: '0600'
    force: true

- name: Dump/Parse PKCS#12 file
  community.crypto.openssl_pkcs12:
    action: parse
    src: /opt/certs/ansible.p12
    path: /opt/certs/ansible.pem
    state: present

- name: Remove PKCS#12 file
  community.crypto.openssl_pkcs12:
    path: /opt/certs/ansible.p12
    state: absent
"""

RETURN = r"""
filename:
  description: Path to the generate PKCS#12 file.
  returned: changed or success
  type: str
  sample: /opt/certs/ansible.p12
privatekey:
  description: Path to the TLS/SSL private key the public key was generated from.
  returned: changed or success
  type: str
  sample: /etc/ssl/private/ansible.com.pem
backup_file:
  description: Name of backup file created.
  returned: changed and if O(backup) is V(true)
  type: str
  sample: /path/to/ansible.com.pem.2019-03-09@11:22~
pkcs12:
  description: The (current or generated) PKCS#12's content Base64 encoded.
  returned: if O(state) is V(present) and O(return_content) is V(true)
  type: str
  version_added: "1.0.0"
"""

import base64
import itertools
import os
import stat
import traceback
import typing as t

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLBadPassphraseError,
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    parse_pkcs12,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.pem import (
    split_pem_list,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    OpenSSLObject,
    load_certificate,
    load_certificate_issuer_privatekey,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)
from ansible_collections.community.crypto.plugins.module_utils._io import (
    load_file_if_exists,
    write_file,
)

MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    from cryptography.hazmat.primitives import serialization
    from cryptography.hazmat.primitives.serialization.pkcs12 import (
        serialize_key_and_certificates,
    )
except ImportError:
    pass

try:
    from cryptography.hazmat.primitives.asymmetric.mldsa import (
        MLDSA44PrivateKey,
        MLDSA65PrivateKey,
        MLDSA87PrivateKey,
    )

    HAS_MLDSA_SUPPORT = True
except ImportError:  # pragma: no cover

    class MLDSA44PrivateKey:  # type: ignore[no-redef]
        pass

    class MLDSA65PrivateKey:  # type: ignore[no-redef]
        pass

    class MLDSA87PrivateKey:  # type: ignore[no-redef]
        pass

    HAS_MLDSA_SUPPORT = False

CRYPTOGRAPHY_COMPATIBILITY2022_ERR: str | None
try:
    import cryptography.x509
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.primitives.serialization.pkcs12 import PBES

    # Try to build encryption builder for compatibility2022
    serialization.PrivateFormat.PKCS12.encryption_builder().key_cert_algorithm(
        PBES.PBESv1SHA1And3KeyTripleDESCBC
    ).hmac_hash(hashes.SHA1())
except Exception:
    # pylint: disable-next=invalid-name
    CRYPTOGRAPHY_COMPATIBILITY2022_ERR = traceback.format_exc()
    CRYPTOGRAPHY_HAS_COMPATIBILITY2022 = False
else:
    CRYPTOGRAPHY_COMPATIBILITY2022_ERR = None  # pylint: disable=invalid-name
    CRYPTOGRAPHY_HAS_COMPATIBILITY2022 = True

if t.TYPE_CHECKING:  # pragma: no cover
    from cryptography.hazmat.primitives.serialization.pkcs12 import (
        PKCS12PrivateKeyTypes,
    )

    PKCS12 = tuple[
        t.Union[PKCS12PrivateKeyTypes, None],  # noqa: UP007
        t.Union[cryptography.x509.Certificate, None],  # noqa: UP007
        list[cryptography.x509.Certificate],
        t.Union[bytes, None],  # noqa: UP007
    ]  # pragma: no cover


def load_certificate_set(
    filename: str | os.PathLike,
) -> list[cryptography.x509.Certificate]:
    """
    Load list of concatenated PEM files, and return a list of parsed certificates.
    """
    with open(filename, "rb") as f:
        data = f.read().decode("utf-8")
    return [
        load_certificate(content=cert.encode("utf-8")) for cert in split_pem_list(data)
    ]


class PkcsError(OpenSSLObjectError):
    pass


class Pkcs(OpenSSLObject):
    path: str

    def __init__(self, module: AnsibleModule, iter_size_default: int = 50000) -> None:
        super().__init__(
            path=module.params["path"],
            state=module.params["state"],
            force=module.params["force"],
            check_mode=module.check_mode,
        )
        self.action: t.Literal["export", "parse"] = module.params["action"]
        self.other_certificates: list[cryptography.x509.Certificate] = []
        self.other_certificates_str: list[str] | None = module.params[
            "other_certificates"
        ]
        self.other_certificates_parse_all: bool = module.params[
            "other_certificates_parse_all"
        ]
        self.other_certificates_content: list[str] | None = module.params[
            "other_certificates_content"
        ]
        self.certificate_path: str | None = module.params["certificate_path"]
        certificate_content: str | None = module.params["certificate_content"]
        self.friendly_name: str | None = module.params["friendly_name"]
        self.iter_size: int = module.params["iter_size"] or iter_size_default
        self.encryption_level: t.Literal["auto", "compatibility2022"] = module.params[
            "encryption_level"
        ]
        self.passphrase = module.params["passphrase"]
        self.pkcs12: PKCS12 | None = None
        self.privatekey_passphrase: str | None = module.params["privatekey_passphrase"]
        self.privatekey_path: str | None = module.params["privatekey_path"]
        privatekey_content: str | None = module.params["privatekey_content"]
        self.pkcs12_bytes: bytes | None = None
        self.return_content: bool = module.params["return_content"]
        self.src: str | None = module.params["src"]

        if module.params["mode"] is None:
            module.params["mode"] = "0400"

        self.backup: bool = module.params["backup"]
        self.backup_file: str | None = None

        self.certificate_content: bytes | None = None
        if self.certificate_path is not None:
            try:
                with open(self.certificate_path, "rb") as fh:
                    self.certificate_content = fh.read()
            except (IOError, OSError) as exc:
                raise PkcsError(exc) from exc
        elif certificate_content is not None:
            self.certificate_content = to_bytes(certificate_content)

        self.privatekey_content: bytes | None = None
        if self.privatekey_path is not None:
            try:
                with open(self.privatekey_path, "rb") as fh:
                    self.privatekey_content = fh.read()
            except (IOError, OSError) as exc:
                raise PkcsError(exc) from exc
        elif privatekey_content is not None:
            self.privatekey_content = to_bytes(privatekey_content)

        if self.other_certificates_str:
            if self.other_certificates_parse_all:
                self.other_certificates = []
                for other_cert_bundle in self.other_certificates_str:
                    self.other_certificates.extend(
                        load_certificate_set(other_cert_bundle)
                    )
            else:
                self.other_certificates = [
                    load_certificate(path=other_cert)
                    for other_cert in self.other_certificates_str
                ]
        elif self.other_certificates_content:
            certs = self.other_certificates_content
            if self.other_certificates_parse_all:
                certs = list(
                    itertools.chain.from_iterable(
                        split_pem_list(content) for content in certs
                    )
                )
            self.other_certificates = [
                load_certificate(content=to_bytes(other_cert)) for other_cert in certs
            ]

        if (
            self.encryption_level == "compatibility2022"
            and not CRYPTOGRAPHY_HAS_COMPATIBILITY2022
        ):
            module.fail_json(
                msg="The installed cryptography version does not support encryption_level = compatibility2022."
                " You need cryptography >= 38.0.0 and support for SHA1",
                exception=CRYPTOGRAPHY_COMPATIBILITY2022_ERR,
            )

    def generate_bytes(self, module: AnsibleModule) -> bytes:
        """Generate PKCS#12 file archive."""
        pkey: PKCS12PrivateKeyTypes | None = None
        if self.privatekey_content:
            try:
                private_key = load_certificate_issuer_privatekey(
                    content=self.privatekey_content,
                    passphrase=self.privatekey_passphrase,
                )
            except OpenSSLBadPassphraseError as exc:
                raise PkcsError(exc) from exc

            if isinstance(
                private_key,
                (
                    MLDSA44PrivateKey,
                    MLDSA65PrivateKey,
                    MLDSA87PrivateKey,
                ),
            ):
                raise PkcsError(f"{private_key} is not a supported private key type")

            pkey = private_key

        cert = None
        if self.certificate_content:
            cert = load_certificate(content=self.certificate_content)

        friendly_name = (
            to_bytes(self.friendly_name) if self.friendly_name is not None else None
        )

        # Store fake object which can be used to retrieve the components back
        self.pkcs12 = (pkey, cert, self.other_certificates, friendly_name)

        encryption: serialization.KeySerializationEncryption
        if not self.passphrase:
            encryption = serialization.NoEncryption()
        elif self.encryption_level == "compatibility2022":
            encryption = (
                serialization.PrivateFormat.PKCS12.encryption_builder()
                .kdf_rounds(self.iter_size)
                .key_cert_algorithm(PBES.PBESv1SHA1And3KeyTripleDESCBC)
                .hmac_hash(hashes.SHA1())
                .build(to_bytes(self.passphrase))
            )
        else:
            encryption = serialization.BestAvailableEncryption(
                to_bytes(self.passphrase)
            )

        return serialize_key_and_certificates(
            friendly_name,
            pkey,
            cert,
            self.other_certificates,
            encryption,
        )

    def parse_bytes(self, pkcs12_content: bytes) -> tuple[
        bytes | None,
        bytes | None,
        list[bytes],
        bytes | None,
    ]:
        try:
            private_key, certificate, additional_certificates, friendly_name = (
                parse_pkcs12(pkcs12_content, passphrase=self.passphrase)
            )

            pkey = None
            if private_key is not None:
                pkey = private_key.private_bytes(
                    encoding=serialization.Encoding.PEM,
                    format=serialization.PrivateFormat.TraditionalOpenSSL,
                    encryption_algorithm=serialization.NoEncryption(),
                )

            crt = None
            if certificate is not None:
                crt = certificate.public_bytes(serialization.Encoding.PEM)

            other_certs = []
            if additional_certificates is not None:
                other_certs = [
                    other_cert.public_bytes(serialization.Encoding.PEM)
                    for other_cert in additional_certificates
                ]

            return (pkey, crt, other_certs, friendly_name)
        except ValueError as exc:
            raise PkcsError(exc) from exc

    def _dump_privatekey(self, pkcs12: PKCS12) -> bytes | None:
        return (
            pkcs12[0].private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption(),
            )
            if pkcs12[0]
            else None
        )

    def _dump_certificate(self, pkcs12: PKCS12) -> bytes | None:
        return pkcs12[1].public_bytes(serialization.Encoding.PEM) if pkcs12[1] else None

    def _dump_other_certificates(self, pkcs12: PKCS12) -> list[bytes]:
        return [
            other_cert.public_bytes(serialization.Encoding.PEM)
            for other_cert in pkcs12[2]
        ]

    def _get_friendly_name(self, pkcs12: PKCS12) -> bytes | None:
        return pkcs12[3]

    def check(self, module: AnsibleModule, *, perms_required: bool = True) -> bool:
        """Ensure the resource is in its desired state."""
        state_and_perms = super().check(module=module, perms_required=perms_required)

        def _check_pkey_passphrase() -> bool:
            if self.privatekey_passphrase:
                try:
                    load_certificate_issuer_privatekey(
                        content=self.privatekey_content,
                        passphrase=self.privatekey_passphrase,
                    )
                except OpenSSLObjectError:
                    return False
            return True

        if not state_and_perms:
            return state_and_perms

        if os.path.exists(self.path) and module.params["action"] == "export":
            self.generate_bytes(module)  # ignore result
            assert self.pkcs12 is not None
            self.src = self.path
            try:
                (
                    pkcs12_privatekey,
                    pkcs12_certificate,
                    pkcs12_other_certificates,
                    pkcs12_friendly_name,
                ) = self.parse()
            except OpenSSLObjectError:
                return False
            if (pkcs12_privatekey is not None) and (
                self.privatekey_content is not None
            ):
                expected_pkey = self._dump_privatekey(self.pkcs12)
                if pkcs12_privatekey != expected_pkey:
                    return False
            elif bool(pkcs12_privatekey) != bool(self.privatekey_content):
                return False

            if (pkcs12_certificate is not None) and (
                self.certificate_content is not None
            ):
                expected_cert = self._dump_certificate(self.pkcs12)
                if pkcs12_certificate != expected_cert:
                    return False
            elif bool(pkcs12_certificate) != bool(self.certificate_content):
                return False

            expected_other_certs = self._dump_other_certificates(self.pkcs12)
            if set(pkcs12_other_certificates) != set(expected_other_certs):
                return False

            if pkcs12_privatekey:
                # This check is required because pyOpenSSL will not return a friendly name
                # if the private key is not set in the file
                friendly_name = self._get_friendly_name(self.pkcs12)
                if (friendly_name is not None) and (pkcs12_friendly_name is not None):
                    if friendly_name != pkcs12_friendly_name:
                        return False
                elif bool(friendly_name) != bool(pkcs12_friendly_name):
                    return False
        elif (
            module.params["action"] == "parse"
            and os.path.exists(self.src or "")
            and os.path.exists(self.path)
        ):
            try:
                pkey, cert, other_certs, friendly_name = self.parse()
            except OpenSSLObjectError:
                return False
            expected_content = to_bytes(
                "".join(
                    [
                        to_text(pem)
                        for pem in [pkey, cert] + other_certs
                        if pem is not None
                    ]
                )
            )
            dumped_content = load_file_if_exists(path=self.path, ignore_errors=True)
            if expected_content != dumped_content:
                return False
        else:
            return False

        return _check_pkey_passphrase()

    def dump(self) -> dict[str, t.Any]:
        """Serialize the object into a dictionary."""

        result: dict[str, t.Any] = {
            "filename": self.path,
        }
        if self.privatekey_path:
            result["privatekey_path"] = self.privatekey_path
        if self.backup_file:
            result["backup_file"] = self.backup_file
        if self.return_content:
            if self.pkcs12_bytes is None:
                self.pkcs12_bytes = load_file_if_exists(
                    path=self.path, ignore_errors=True
                )
            result["pkcs12"] = (
                base64.b64encode(self.pkcs12_bytes) if self.pkcs12_bytes else None
            )

        return result

    def remove(self, module: AnsibleModule) -> None:
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        super().remove(module)

    def parse(
        self,
    ) -> tuple[
        bytes | None,
        bytes | None,
        list[bytes],
        bytes | None,
    ]:
        """Read PKCS#12 file."""
        if self.src is None:
            raise AssertionError("Contract violation: src is None")  # pragma: no cover

        try:
            with open(self.src, "rb") as pkcs12_fh:
                pkcs12_content = pkcs12_fh.read()
            return self.parse_bytes(pkcs12_content)
        except IOError as exc:
            raise PkcsError(exc) from exc

    def generate(self, module: AnsibleModule) -> None:
        # Empty method because OpenSSLObject wants this
        pass

    def write(
        self, module: AnsibleModule, content: bytes, mode: int | str | None = None
    ) -> None:
        """Write the PKCS#12 file."""
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        write_file(module=module, content=content, default_mode=mode)
        if self.return_content:
            self.pkcs12_bytes = content


def select_backend(module: AnsibleModule) -> Pkcs:
    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )
    return Pkcs(module)


def main() -> t.NoReturn:
    argument_spec = {
        "action": {"type": "str", "default": "export", "choices": ["export", "parse"]},
        "other_certificates": {
            "type": "list",
            "elements": "path",
            "aliases": ["ca_certificates"],
        },
        "other_certificates_parse_all": {"type": "bool", "default": False},
        "other_certificates_content": {"type": "list", "elements": "str"},
        "certificate_path": {"type": "path"},
        "certificate_content": {"type": "str"},
        "force": {"type": "bool", "default": False},
        "friendly_name": {"type": "str", "aliases": ["name"]},
        "encryption_level": {
            "type": "str",
            "choices": ["auto", "compatibility2022"],
            "default": "auto",
        },
        "iter_size": {"type": "int"},
        "maciter_size": {
            "type": "int",
            "removed_in_version": "4.0.0",
            "removed_from_collection": "community.crypto",
        },
        "passphrase": {"type": "str", "no_log": True},
        "path": {"type": "path", "required": True},
        "privatekey_passphrase": {"type": "str", "no_log": True},
        "privatekey_path": {"type": "path"},
        "privatekey_content": {"type": "str", "no_log": True},
        "state": {
            "type": "str",
            "default": "present",
            "choices": ["absent", "present"],
        },
        "src": {"type": "path"},
        "backup": {"type": "bool", "default": False},
        "return_content": {"type": "bool", "default": False},
        "select_crypto_backend": {
            "type": "str",
            "default": "auto",
            "choices": ["auto", "cryptography"],
        },
    }

    required_if = [
        ["action", "parse", ["src"]],
    ]

    mutually_exclusive = [
        ["privatekey_path", "privatekey_content"],
        ["certificate_path", "certificate_content"],
        ["other_certificates", "other_certificates_content"],
    ]

    module = AnsibleModule(
        add_file_common_args=True,
        argument_spec=argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    pkcs12 = select_backend(module)

    base_dir = os.path.dirname(module.params["path"]) or "."
    if not os.path.isdir(base_dir):
        module.fail_json(
            name=base_dir,
            msg=f"The directory '{base_dir}' does not exist or the path is not a directory",
        )

    try:
        changed = False

        if module.params["state"] == "present":
            if module.check_mode:
                result = pkcs12.dump()
                result["changed"] = module.params["force"] or not pkcs12.check(module)
                module.exit_json(**result)

            if not pkcs12.check(module, perms_required=False) or module.params["force"]:
                if module.params["action"] == "export":
                    if not module.params["friendly_name"]:
                        module.fail_json(msg="Friendly_name is required")
                    pkcs12_content = pkcs12.generate_bytes(module)
                    pkcs12.write(module, pkcs12_content, 0o600)
                    changed = True
                else:
                    pkey, cert, other_certs, _friendly_name = pkcs12.parse()
                    dump_content = "".join(
                        [
                            to_text(pem)
                            for pem in [pkey, cert] + other_certs
                            if pem is not None
                        ]
                    )
                    pkcs12.write(module, to_bytes(dump_content))
                    changed = True

            file_args = module.load_file_common_arguments(module.params)
            if module.check_file_absent_if_check_mode(
                file_args["path"]
            ) or module.set_fs_attributes_if_different(file_args, changed):
                changed = True
        else:
            if module.check_mode:
                result = pkcs12.dump()
                result["changed"] = os.path.exists(module.params["path"])
                module.exit_json(**result)

            if os.path.exists(module.params["path"]):
                pkcs12.remove(module)
                changed = True

        result = pkcs12.dump()
        result["changed"] = changed
        if os.path.exists(module.params["path"]):
            file_mode = f"{stat.S_IMODE(os.stat(module.params['path']).st_mode):04o}"
            result["mode"] = file_mode

        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()
